@@ -22,6 +22,12 @@
 #ifndef SVN_MERGEINFO_PRIVATE_H
 #define SVN_MERGEINFO_PRIVATE_H
 
+#include <apr_pools.h>
+
+#include "svn_types.h"
+#include "svn_error.h"
+#include "svn_mergeinfo.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
@@ -33,17 +39,17 @@ extern "C" {
    hashes are compared for equality.  If CONSIDER_INERITANCE is FALSE,
    then the start and end revisions of the svn_merge_range_t's being
    compared are the only factors considered when determining equality.
-  
+
      e.g. '/trunk: 1,3-4*,5' == '/trunk: 1,3-5'
- 
+
    If CONSIDER_INERITANCE is TRUE, then the inheritability of the
    svn_merge_range_t's is also considered and must be the same for two
    otherwise identical ranges to be judged equal.
- 
+
      e.g. '/trunk: 1,3-4*,5' != '/trunk: 1,3-5'
           '/trunk: 1,3-4*,5' == '/trunk: 1,3-4*,5'
           '/trunk: 1,3-4,5'  == '/trunk: 1,3-4,5'
- 
+
    Use POOL for temporary allocations. */
 svn_error_t *
 svn_mergeinfo__equals(svn_boolean_t *is_equal,
@@ -79,8 +85,9 @@ svn_mergeinfo__remove_prefix_from_catalog(svn_mergeinfo_catalog_t *out_catalog,
    If CATALOG is empty or NULL then *OUTPUT->DATA is set to "\n".  If SVN_DEBUG
    is true, then a NULL or empty CATALOG causes *OUTPUT to be set to an
    appropriate newline terminated string.  If KEY_PREFIX is not NULL then
-   KEY_PREFIX to each key (path) in *OUTPUT.  if VAL_PREFIX is not NULL then
-   prepend VAL_PREFIX to each merge source:rangelist line in *OUTPUT. */
+   prepend KEY_PREFIX to each key (path) in *OUTPUT.  if VAL_PREFIX is not
+   NULL then prepend VAL_PREFIX to each merge source:rangelist line in
+   *OUTPUT. */
 svn_error_t *
 svn_mergeinfo__catalog_to_formatted_string(svn_string_t **output,
                                            svn_mergeinfo_catalog_t catalog,
@@ -111,7 +118,7 @@ svn_mergeinfo__get_range_endpoints(svn_revnum_t *youngest_rev,
 /* Set *FILTERED_MERGEINFO to a deep copy of MERGEINFO, allocated in POOL, less
    any rangelists that fall outside of the range OLDEST_REV:YOUGEST_REV
    (inclusive).  If all the rangelists mapped to a given path are filtered
-   then filter that path as well.  If all paths are filtered or MERGEINFO is 
+   then filter that path as well.  If all paths are filtered or MERGEINFO is
    empty or NULL then *FILTERED_MERGEINFO is set to an empty hash. */
 svn_error_t *
 svn_mergeinfo__filter_mergeinfo_by_ranges(svn_mergeinfo_t *filtered_mergeinfo,
@@ -133,10 +140,6 @@ svn_mergeinfo__filter_catalog_by_ranges(
   svn_revnum_t oldest_rev,
   apr_pool_t *pool);
 
-/* Return a deep copy of MERGEINFO_CATALOG, allocated in POOL. */
-svn_mergeinfo_catalog_t
-svn_mergeinfo__catalog_dup(svn_mergeinfo_catalog_t mergeinfo_catalog,
-                           apr_pool_t *pool);
 
 #ifdef __cplusplus
 }
