@@ -2,7 +2,7 @@
  * add.c:  wrappers around wc add/mkdir functionality.
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -30,6 +30,7 @@
 #include "svn_string.h"
 #include "svn_pools.h"
 #include "svn_error.h"
+#include "svn_dirent_uri.h"
 #include "svn_path.h"
 #include "svn_io.h"
 #include "svn_config.h"
@@ -186,10 +187,6 @@ svn_client__get_auto_props(apr_hash_t **properties,
                      svn_string_create(autoprops.mimetype, pool));
     }
 
-  /* Don't automatically set the svn:executable property on added items
-   * on OS400.  While OS400 supports the executable permission its use is
-   * inconsistent at best. */
-#ifndef AS400
   /* if executable has not been set check the file */
   if (! autoprops.have_executable)
     {
@@ -200,7 +197,6 @@ svn_client__get_auto_props(apr_hash_t **properties,
                      strlen(SVN_PROP_EXECUTABLE),
                      svn_string_create("", pool));
     }
-#endif
 
   *mimetype = autoprops.mimetype;
   return SVN_NO_ERROR;
@@ -232,16 +228,16 @@ add_file(const char *path,
                                        pool));
 
   /* Add the file */
-  SVN_ERR(svn_wc_add2(path, adm_access, NULL, SVN_INVALID_REVNUM,
-                      ctx->cancel_func, ctx->cancel_baton,
+  SVN_ERR(svn_wc_add3(path, adm_access, svn_depth_infinity, NULL,
+                      SVN_INVALID_REVNUM, ctx->cancel_func, ctx->cancel_baton,
                       NULL, NULL, pool));
 
   if (is_special)
     /* This must be a special file. */
-    SVN_ERR(svn_wc_prop_set2
+    SVN_ERR(svn_wc_prop_set3
             (SVN_PROP_SPECIAL,
              svn_string_create(SVN_PROP_BOOLEAN_TRUE, pool),
-             path, adm_access, FALSE, pool));
+             path, adm_access, FALSE, NULL, NULL, pool));
   else if (properties)
     {
       /* loop through the hashtable and add the properties */
@@ -255,8 +251,8 @@ add_file(const char *path,
           /* It's probably best to pass 0 for force, so that if
              the autoprops say to set some weird combination,
              we just error and let the user sort it out. */
-          SVN_ERR(svn_wc_prop_set2(pname, pval, path,
-                                   adm_access, FALSE, pool));
+          SVN_ERR(svn_wc_prop_set3(pname, pval, path, adm_access, FALSE,
+                                   NULL, NULL, pool));
         }
     }
 
@@ -308,8 +304,7 @@ add_dir_recursive(const char *dirname,
     SVN_ERR(ctx->cancel_func(ctx->cancel_baton));
 
   /* Add this directory to revision control. */
-  err = svn_wc_add2(dirname, adm_access,
-                    NULL, SVN_INVALID_REVNUM,
+  err = svn_wc_add3(dirname, adm_access, depth, NULL, SVN_INVALID_REVNUM,
                     ctx->cancel_func, ctx->cancel_baton,
                     ctx->notify_func2, ctx->notify_baton2, pool);
   if (err && err->apr_err == SVN_ERR_ENTRY_EXISTS && force)
@@ -405,7 +400,7 @@ add_dir_recursive(const char *dirname,
     }
 
   /* Opened by svn_wc_add */
-  SVN_ERR(svn_wc_adm_close(dir_access));
+  SVN_ERR(svn_wc_adm_close2(dir_access, subpool));
 
   /* Destroy the per-iteration pool. */
   svn_pool_destroy(subpool);
@@ -430,13 +425,18 @@ add(const char *path,
   svn_error_t *err;
 
   SVN_ERR(svn_io_check_path(path, &kind, pool));
-  if (kind == svn_node_dir && depth >= svn_depth_files)
-    err = add_dir_recursive(path, adm_access, depth,
-                            force, no_ignore, ctx, pool);
+  if (kind == svn_node_dir)
+    {
+      /* We use add_dir_recursive for all directory targets
+         and pass depth along no matter what it is, so that the
+         target's depth will be set correctly. */
+      err = add_dir_recursive(path, adm_access, depth,
+                              force, no_ignore, ctx, pool);
+    }
   else if (kind == svn_node_file)
     err = add_file(path, ctx, adm_access, pool);
   else
-    err = svn_wc_add2(path, adm_access, NULL, SVN_INVALID_REVNUM,
+    err = svn_wc_add3(path, adm_access, depth, NULL, SVN_INVALID_REVNUM,
                       ctx->cancel_func, ctx->cancel_baton,
                       ctx->notify_func2, ctx->notify_baton2, pool);
 
@@ -477,7 +477,7 @@ add_parent_dirs(const char *path,
         {
           return svn_error_createf
             (SVN_ERR_RESERVED_FILENAME_SPECIFIED, NULL,
-             _("'%s' ends in a reserved name"), 
+             _("'%s' ends in a reserved name"),
              svn_path_local_style(path, pool));
         }
       else
@@ -487,7 +487,8 @@ add_parent_dirs(const char *path,
           SVN_ERR(add_parent_dirs(parent_path, &adm_access, ctx, pool));
           SVN_ERR(svn_wc_adm_retrieve(&adm_access, adm_access, parent_path,
                                       pool));
-          SVN_ERR(svn_wc_add2(path, adm_access, NULL, SVN_INVALID_REVNUM,
+          SVN_ERR(svn_wc_add3(path, adm_access, svn_depth_infinity,
+                              NULL, SVN_INVALID_REVNUM,
                               ctx->cancel_func, ctx->cancel_baton,
                               ctx->notify_func2, ctx->notify_baton2, pool));
         }
@@ -527,24 +528,21 @@ svn_client_add4(const char *path,
 
       subpool = svn_pool_create(pool);
       SVN_ERR(add_parent_dirs(parent_dir, &adm_access, ctx, subpool));
-      SVN_ERR(svn_wc_adm_close(adm_access));
+      SVN_ERR(svn_wc_adm_close2(adm_access, subpool));
       svn_pool_destroy(subpool);
-
-      SVN_ERR(svn_wc_adm_open3(&adm_access, NULL, parent_dir,
-                               TRUE, 0, ctx->cancel_func, ctx->cancel_baton,
-                               pool));
     }
   else
     {
       parent_dir = svn_path_dirname(path, pool);
-      SVN_ERR(svn_wc_adm_open3(&adm_access, NULL, parent_dir,
-                               TRUE, 0, ctx->cancel_func, ctx->cancel_baton,
-                               pool));
     }
 
+  SVN_ERR(svn_wc_adm_open3(&adm_access, NULL, parent_dir,
+                           TRUE, 0, ctx->cancel_func, ctx->cancel_baton,
+                           pool));
+
   err = add(path, depth, force, no_ignore, adm_access, ctx, pool);
 
-  err2 = svn_wc_adm_close(adm_access);
+  err2 = svn_wc_adm_close2(adm_access, pool);
   if (err2)
     {
       if (err)
@@ -556,38 +554,6 @@ svn_client_add4(const char *path,
   return err;
 }
 
-svn_error_t *
-svn_client_add3(const char *path,
-                svn_boolean_t recursive,
-                svn_boolean_t force,
-                svn_boolean_t no_ignore,
-                svn_client_ctx_t *ctx,
-                apr_pool_t *pool)
-{
-  return svn_client_add4(path, SVN_DEPTH_INFINITY_OR_EMPTY(recursive),
-                         force, no_ignore, FALSE, ctx,
-                         pool);
-}
-
-svn_error_t *
-svn_client_add2(const char *path,
-                svn_boolean_t recursive,
-                svn_boolean_t force,
-                svn_client_ctx_t *ctx,
-                apr_pool_t *pool)
-{
-  return svn_client_add3(path, recursive, force, FALSE, ctx, pool);
-}
-
-svn_error_t *
-svn_client_add(const char *path,
-               svn_boolean_t recursive,
-               svn_client_ctx_t *ctx,
-               apr_pool_t *pool)
-{
-  return svn_client_add3(path, recursive, FALSE, FALSE, ctx, pool);
-}
-
 
 static svn_error_t *
 path_driver_cb_func(void **dir_baton,
@@ -713,7 +679,7 @@ mkdir_urls(svn_commit_info_t **commit_info_p,
             }
         }
     }
-  qsort(targets->elts, targets->nelts, targets->elt_size, 
+  qsort(targets->elts, targets->nelts, targets->elt_size,
         svn_sort_compare_paths);
 
   /* Create new commit items and add them to the array. */
@@ -727,8 +693,8 @@ mkdir_urls(svn_commit_info_t **commit_info_p,
       for (i = 0; i < targets->nelts; i++)
         {
           const char *path = APR_ARRAY_IDX(targets, i, const char *);
-          SVN_ERR(svn_client_commit_item_create
-                  ((const svn_client_commit_item3_t **) &item, pool));
+
+          item = svn_client_commit_item3_create(pool);
           item->url = svn_path_join(common, path, pool);
           item->state_flags = SVN_CLIENT_COMMIT_ITEM_ADD;
           APR_ARRAY_PUSH(commit_items, svn_client_commit_item3_t *) = item;
@@ -782,9 +748,7 @@ mkdir_urls(svn_commit_info_t **commit_info_p,
     }
 
   /* Close the edit. */
-  SVN_ERR(editor->close_edit(edit_baton, pool));
-
-  return SVN_NO_ERROR;
+  return editor->close_edit(edit_baton, pool);
 }
 
 
@@ -803,18 +767,22 @@ svn_client__make_local_parents(const char *path,
   else
     SVN_ERR(svn_io_dir_make(path, APR_OS_DEFAULT, pool));
 
-  err = svn_client_add4(path, svn_depth_empty, FALSE, FALSE,
+  /* Should no longer use svn_depth_empty to indicate that only the directory
+     itself is added, since it not only constraints the operation depth, but
+     also defines the depth of the target directory now. Moreover, the new
+     directory will have no children at all.*/
+  err = svn_client_add4(path, svn_depth_infinity, FALSE, FALSE,
                         make_parents, ctx, pool);
 
-  /* We just created a new directory, but couldn't add it to
-     version control. Don't leave unversioned directories behind. */
+  /* If we created a new directory, but couldn't add it to version
+     control, then delete it. */
   if (err && (orig_kind == svn_node_none))
     {
       /* ### If this returns an error, should we link it onto
          err instead, so that the user is warned that we just
          created an unversioned directory? */
 
-      svn_error_clear(svn_io_remove_dir(path, pool));
+      svn_error_clear(svn_io_remove_dir2(path, FALSE, NULL, NULL, pool));
     }
 
   return err;
@@ -834,7 +802,7 @@ svn_client_mkdir3(svn_commit_info_t **commit_info_p,
 
   if (svn_path_is_url(APR_ARRAY_IDX(paths, 0, const char *)))
     {
-      SVN_ERR(mkdir_urls(commit_info_p, paths, make_parents, 
+      SVN_ERR(mkdir_urls(commit_info_p, paths, make_parents,
                          revprop_table, ctx, pool));
     }
   else
@@ -861,29 +829,3 @@ svn_client_mkdir3(svn_commit_info_t **commit_info_p,
 
   return SVN_NO_ERROR;
 }
-
-
-svn_error_t *
-svn_client_mkdir2(svn_commit_info_t **commit_info_p,
-                  const apr_array_header_t *paths,
-                  svn_client_ctx_t *ctx,
-                  apr_pool_t *pool)
-{
-  return svn_client_mkdir3(commit_info_p, paths, FALSE, NULL, ctx, pool);
-}
-
-
-svn_error_t *
-svn_client_mkdir(svn_client_commit_info_t **commit_info_p,
-                 const apr_array_header_t *paths,
-                 svn_client_ctx_t *ctx,
-                 apr_pool_t *pool)
-{
-  svn_commit_info_t *commit_info = NULL;
-  svn_error_t *err;
-
-  err = svn_client_mkdir2(&commit_info, paths, ctx, pool);
-  /* These structs have the same layout for the common fields. */
-  *commit_info_p = (svn_client_commit_info_t *) commit_info;
-  return err;
-}
