@@ -18,9 +18,9 @@
 
 #include <apr_pools.h>
 #include <apr_strings.h>
-#include <assert.h>
 
 #include "svn_pools.h"
+#include "svn_dirent_uri.h"
 #include "svn_path.h"
 #include "svn_string.h"
 #include "svn_opt.h"
@@ -42,6 +42,25 @@
 
 
 
+svn_client__merge_path_t *
+svn_client__merge_path_dup(const svn_client__merge_path_t *old,
+                           apr_pool_t *pool)
+{
+  svn_client__merge_path_t *new = apr_pmemdup(pool, old, sizeof(*old));
+
+  new->path = apr_pstrdup(pool, old->path);
+  if (new->remaining_ranges)
+    new->remaining_ranges = svn_rangelist_dup(old->remaining_ranges, pool);
+  if (new->pre_merge_mergeinfo)
+    new->pre_merge_mergeinfo = svn_mergeinfo_dup(old->pre_merge_mergeinfo,
+                                                 pool);
+  if (new->implicit_mergeinfo)
+    new->implicit_mergeinfo = svn_mergeinfo_dup(old->implicit_mergeinfo,
+                                                 pool);
+
+  return new;
+}
+
 svn_error_t *
 svn_client__parse_mergeinfo(svn_mergeinfo_t *mergeinfo,
                             const svn_wc_entry_t *entry,
@@ -62,11 +81,12 @@ svn_client__parse_mergeinfo(svn_mergeinfo_t *mergeinfo,
                                        svn_depth_empty, NULL, ctx, pool));
   propval = apr_hash_get(props, wcpath, APR_HASH_KEY_STRING);
   if (propval)
-    SVN_ERR(svn_mergeinfo_parse(mergeinfo, propval->data, pool));
+    return svn_mergeinfo_parse(mergeinfo, propval->data, pool);
   else
-    *mergeinfo = NULL;
-
-  return SVN_NO_ERROR;
+    {
+      *mergeinfo = NULL;
+      return SVN_NO_ERROR;
+    }
 }
 
 svn_error_t *
@@ -92,8 +112,8 @@ svn_client__record_wc_mergeinfo(const char *wcpath,
   /* Record the new mergeinfo in the WC. */
   /* ### Later, we'll want behavior more analogous to
      ### svn_client__get_prop_from_wc(). */
-  return svn_wc_prop_set2(SVN_PROP_MERGEINFO, mergeinfo_str, wcpath,
-                          adm_access, TRUE /* skip checks */, pool);
+  return svn_wc_prop_set3(SVN_PROP_MERGEINFO, mergeinfo_str, wcpath,
+                          adm_access, TRUE /* skip checks */, NULL, NULL, pool);
 }
 
 /*-----------------------------------------------------------------------*/
@@ -176,21 +196,10 @@ svn_client__get_wc_mergeinfo(svn_mergeinfo_t *mergeinfo,
          an absolute path so we can walk up and out of the WC
          if necessary.  If we are using LIMIT_PATH it needs to
          be absolute too. */
-#if defined(WIN32) || defined(__CYGWIN__)
-      /* On Windows a path is also absolute when it starts with
-         'H:/' where 'H' is any upper or lower case letter. */
-      if (strlen(wcpath) == 0
-          || ((strlen(wcpath) > 0 && wcpath[0] != '/')
-               && !(strlen(wcpath) > 2
-                    && wcpath[1] == ':'
-                    && wcpath[2] == '/'
-                    && ((wcpath[0] >= 'A' && wcpath[0] <= 'Z')
-                        || (wcpath[0] >= 'a' && wcpath[0] <= 'z')))))
-#else
-      if (!(strlen(wcpath) > 0 && wcpath[0] == '/'))
-#endif /* WIN32 or Cygwin */
+
+      if (! svn_dirent_is_absolute(wcpath))
         {
-          SVN_ERR(svn_path_get_absolute(&wcpath, wcpath, pool));
+          SVN_ERR(svn_dirent_get_absolute(&wcpath, wcpath, pool));
         }
 
       if (wc_mergeinfo == NULL &&
@@ -295,7 +304,7 @@ svn_client__get_repos_mergeinfo(svn_ra_session_t *ra_session,
   /* Temporarily point the session at the root of the repository. */
   SVN_ERR(svn_client__ensure_ra_session_url(&old_session_url, ra_session,
                                             NULL, pool));
-  
+
   /* Fetch the mergeinfo. */
   err = svn_ra_get_mergeinfo(ra_session, &repos_mergeinfo, rel_paths, rev,
                              inherit, FALSE, pool);
@@ -340,7 +349,7 @@ svn_client__get_wc_or_repos_mergeinfo(svn_mergeinfo_t *target_mergeinfo,
   const char *url;
   svn_revnum_t target_rev;
 
-  /* We may get an entry with abrieviated information from TARGET_WCPATH's
+  /* We may get an entry with abbreviated information from TARGET_WCPATH's
      parent if TARGET_WCPATH is missing.  These limited entries do not have
      a URL and without that we cannot get accurate mergeinfo for
      TARGET_WCPATH. */
@@ -354,12 +363,12 @@ svn_client__get_wc_or_repos_mergeinfo(svn_mergeinfo_t *target_mergeinfo,
                                          inherit, entry, target_wcpath,
                                          NULL, NULL, adm_access, ctx, pool));
 
-  /* If there in no WC mergeinfo check the repository. */
+  /* If there is no WC mergeinfo check the repository. */
   if (*target_mergeinfo == NULL)
     {
       svn_mergeinfo_t repos_mergeinfo;
 
-      /* No need to check the repos is this is a local addition. */
+      /* No need to check the repos if this is a local addition. */
       if (entry->schedule != svn_wc_schedule_add)
         {
           apr_hash_t *props = apr_hash_make(pool);
@@ -383,9 +392,9 @@ svn_client__get_wc_or_repos_mergeinfo(svn_mergeinfo_t *target_mergeinfo,
                                                              FALSE, TRUE, ctx,
                                                              pool));
 
-              SVN_ERR(svn_client__path_relative_to_root(&repos_rel_path, url, 
+              SVN_ERR(svn_client__path_relative_to_root(&repos_rel_path, url,
                                                         entry->repos, FALSE,
-                                                        ra_session, NULL, 
+                                                        ra_session, NULL,
                                                         pool));
               SVN_ERR(svn_client__get_repos_mergeinfo(ra_session,
                                                       &repos_mergeinfo,
@@ -417,7 +426,7 @@ svn_client__mergeinfo_from_segments(svn_mergeinfo_t *mergeinfo_p,
   /* Translate location segments into merge sources and ranges. */
   for (i = 0; i < segments->nelts; i++)
     {
-      svn_location_segment_t *segment = 
+      svn_location_segment_t *segment =
         APR_ARRAY_IDX(segments, i, svn_location_segment_t *);
       apr_array_header_t *path_ranges;
       svn_merge_range_t *range;
@@ -489,8 +498,8 @@ svn_client__get_history_as_mergeinfo(svn_mergeinfo_t *mergeinfo_p,
     range_youngest = peg_revnum;
   if (! SVN_IS_VALID_REVNUM(range_oldest))
     range_oldest = 0;
-  SVN_ERR(svn_client__repos_location_segments(&segments, session, "", 
-                                              peg_revnum, range_youngest, 
+  SVN_ERR(svn_client__repos_location_segments(&segments, session, "",
+                                              peg_revnum, range_youngest,
                                               range_oldest, ctx, pool));
 
   SVN_ERR(svn_client__mergeinfo_from_segments(mergeinfo_p, segments, pool));
@@ -518,12 +527,12 @@ svn_client__get_history_as_mergeinfo(svn_mergeinfo_t *mergeinfo_p,
    i.e. if it is NULL then the caller not only walked the entire WC
    looking for inherited mergeinfo, but queried the repository if none
    was found in the WC.  This is rather important since this function
-   says empty mergeinfo mergeinfo should be elided if PARENT_MERGEINFO
-   is NULL, and we don't want to do that unless we are *certain* that
-   the empty mergeinfo on PATH isn't overriding anything.
-   
-   If PATH_SUFFIX and PARENT_MERGEINFO are not NULL append PATH_SUFFIX to each
-   path in PARENT_MERGEINFO before performing the comparison. */
+   says empty mergeinfo should be elided if PARENT_MERGEINFO is NULL,
+   and we don't want to do that unless we are *certain* that the empty
+   mergeinfo on PATH isn't overriding anything.
+
+   If PATH_SUFFIX and PARENT_MERGEINFO are not NULL append PATH_SUFFIX
+   to each path in PARENT_MERGEINFO before performing the comparison. */
 static svn_error_t *
 should_elide_mergeinfo(svn_boolean_t *elides,
                        svn_mergeinfo_t parent_mergeinfo,
@@ -560,7 +569,7 @@ should_elide_mergeinfo(svn_boolean_t *elides,
       apr_pool_t *subpool = svn_pool_create(pool);
 
       path_tweaked_parent_mergeinfo = apr_hash_make(subpool);
-      
+
       /* If we need to adjust the paths in PARENT_MERGEINFO do it now. */
       if (path_suffix)
         adjust_mergeinfo_source_paths(path_tweaked_parent_mergeinfo,
@@ -582,9 +591,9 @@ should_elide_mergeinfo(svn_boolean_t *elides,
 
    Given a working copy PATH, its mergeinfo hash CHILD_MERGEINFO, and
    the mergeinfo of PATH's nearest ancestor PARENT_MERGEINFO, use
-   should_elide_mergeinfo() to decide whether or not CHILD_MERGEINFO elides to 
+   should_elide_mergeinfo() to decide whether or not CHILD_MERGEINFO elides to
    PARENT_MERGEINFO; PATH_SUFFIX means the same as in that function.
-   
+
    If elision does occur, then update the mergeinfo for PATH (which is
    the child) in the working copy via ADM_ACCESS appropriately.
 
@@ -604,8 +613,8 @@ elide_mergeinfo(svn_mergeinfo_t parent_mergeinfo,
                                  path_suffix, pool));
 
   if (elides)
-    SVN_ERR(svn_wc_prop_set2(SVN_PROP_MERGEINFO, NULL, path, adm_access,
-                             TRUE, pool));
+    SVN_ERR(svn_wc_prop_set3(SVN_PROP_MERGEINFO, NULL, path, adm_access,
+                             TRUE, NULL, NULL, pool));
 
   return SVN_NO_ERROR;
 }
@@ -843,7 +852,7 @@ get_mergeinfo(svn_mergeinfo_t *mergeinfo,
                                               peg_revision, "", subpool));
       SVN_ERR(svn_ra_get_repos_root2(ra_session, repos_root, pool));
       SVN_ERR(svn_client__path_relative_to_root(&repos_rel_path, path_or_url,
-                                                *repos_root, FALSE, NULL, 
+                                                *repos_root, FALSE, NULL,
                                                 NULL, subpool));
       SVN_ERR(svn_client__get_repos_mergeinfo(ra_session, mergeinfo,
                                               repos_rel_path, rev,
@@ -881,7 +890,7 @@ get_mergeinfo(svn_mergeinfo_t *mergeinfo,
                                                     svn_mergeinfo_inherited,
                                                     NULL, path_or_url,
                                                     adm_access, ctx, pool));
-      SVN_ERR(svn_wc_adm_close(adm_access));
+      SVN_ERR(svn_wc_adm_close2(adm_access, subpool));
     }
 
   svn_pool_destroy(subpool);
@@ -905,7 +914,7 @@ elide_mergeinfo_catalog_open_root(void *eb,
                                   apr_pool_t *dir_pool,
                                   void **root_baton)
 {
-  struct elide_mergeinfo_catalog_dir_baton *b = apr_pcalloc(dir_pool, 
+  struct elide_mergeinfo_catalog_dir_baton *b = apr_pcalloc(dir_pool,
                                                             sizeof(*b));
   b->mergeinfo_catalog = eb;
   *root_baton = b;
@@ -923,7 +932,7 @@ elide_mergeinfo_catalog_open_directory(const char *path,
                                        void **child_baton)
 {
   struct elide_mergeinfo_catalog_dir_baton *b, *pb = parent_baton;
-  
+
   b = apr_pcalloc(dir_pool, sizeof(*b));
   b->mergeinfo_catalog = pb->mergeinfo_catalog;
 
@@ -931,7 +940,7 @@ elide_mergeinfo_catalog_open_directory(const char *path,
     b->inherited_mergeinfo_path = apr_pstrdup(dir_pool, path);
   else
     b->inherited_mergeinfo_path = pb->inherited_mergeinfo_path;
-  
+
   *child_baton = b;
   return SVN_NO_ERROR;
 }
@@ -958,7 +967,7 @@ elide_mergeinfo_catalog_cb(void **dir_baton,
 
   /* pb == NULL would imply that there was an *empty* path in the
      paths given to the driver (which is different from "/"). */
-  assert(pb != NULL);
+  SVN_ERR_ASSERT(pb != NULL);
 
   /* We'll just act like everything is a file. */
   *dir_baton = NULL;
@@ -972,7 +981,7 @@ elide_mergeinfo_catalog_cb(void **dir_baton,
 
   path_suffix = svn_path_is_child(pb->inherited_mergeinfo_path,
                                   path, NULL);
-  assert(path_suffix != NULL);
+  SVN_ERR_ASSERT(path_suffix != NULL);
 
   SVN_ERR(should_elide_mergeinfo(&elides,
                                  apr_hash_get(cb->mergeinfo_catalog,
@@ -996,7 +1005,7 @@ svn_client__elide_mergeinfo_catalog(svn_mergeinfo_t mergeinfo_catalog,
                                     apr_pool_t *pool)
 {
   apr_array_header_t *paths;
-  apr_array_header_t *elidable_paths = apr_array_make(pool, 1, 
+  apr_array_header_t *elidable_paths = apr_array_make(pool, 1,
                                                       sizeof(const char *));
   svn_delta_editor_t *editor = svn_delta_default_editor(pool);
   struct elide_mergeinfo_catalog_cb_baton cb = {elidable_paths,
@@ -1056,12 +1065,12 @@ filter_log_entry_with_rangelist(void *baton,
   range->end = log_entry->revision;
   range->inheritable = TRUE;
   APR_ARRAY_PUSH(this_rangelist, svn_merge_range_t *) = range;
-  SVN_ERR(svn_rangelist_intersect(&intersection, fleb->rangelist, 
+  SVN_ERR(svn_rangelist_intersect(&intersection, fleb->rangelist,
                                   this_rangelist, TRUE, pool));
   if (! (intersection && intersection->nelts))
     return SVN_NO_ERROR;
 
-  assert (intersection->nelts == 1);
+  SVN_ERR_ASSERT(intersection->nelts == 1);
   return fleb->log_receiver(fleb->log_receiver_baton, log_entry, pool);
 }
 
@@ -1077,14 +1086,16 @@ logs_for_mergeinfo_rangelist(const char *source_url,
 {
   apr_array_header_t *target;
   svn_merge_range_t *oldest_range, *youngest_range;
+  apr_array_header_t *revision_ranges;
   svn_opt_revision_t oldest_rev, youngest_rev;
+  svn_opt_revision_range_t *range;
   struct filter_log_entry_baton_t fleb;
 
   if (! rangelist->nelts)
     return SVN_NO_ERROR;
 
   /* Sort the rangelist. */
-  qsort(rangelist->elts, rangelist->nelts, 
+  qsort(rangelist->elts, rangelist->nelts,
         rangelist->elt_size, svn_sort_compare_ranges);
 
   /* Build a single-member log target list using SOURCE_URL. */
@@ -1092,7 +1103,7 @@ logs_for_mergeinfo_rangelist(const char *source_url,
   APR_ARRAY_PUSH(target, const char *) = source_url;
 
   /* Calculate and construct the bounds of our log request. */
-  youngest_range = APR_ARRAY_IDX(rangelist, rangelist->nelts - 1, 
+  youngest_range = APR_ARRAY_IDX(rangelist, rangelist->nelts - 1,
                                  svn_merge_range_t *);
   youngest_rev.kind = svn_opt_revision_number;
   youngest_rev.value.number = youngest_range->end;
@@ -1107,8 +1118,13 @@ logs_for_mergeinfo_rangelist(const char *source_url,
   fleb.ctx = ctx;
 
   /* Drive the log. */
-  SVN_ERR(svn_client_log4(target, &youngest_rev, &oldest_rev, &youngest_rev, 
-                          0, discover_changed_paths, FALSE, FALSE, revprops, 
+  revision_ranges = apr_array_make(pool, 1, sizeof(svn_opt_revision_range_t *));
+  range = apr_pcalloc(pool, sizeof(*range));
+  range->end = youngest_rev;
+  range->start = oldest_rev;
+  APR_ARRAY_PUSH(revision_ranges, svn_opt_revision_range_t *) = range;
+  SVN_ERR(svn_client_log5(target, &youngest_rev, revision_ranges,
+                          0, discover_changed_paths, FALSE, FALSE, revprops,
                           filter_log_entry_with_rangelist, &fleb, ctx, pool));
 
   /* Check for cancellation. */
@@ -1150,9 +1166,7 @@ location_from_path_and_rev(const char **url,
   svn_revnum_t rev;
 
   if (! svn_path_is_url(path_or_url)
-      && (peg_revision->kind == svn_opt_revision_base
-          || peg_revision->kind == svn_opt_revision_working
-          || peg_revision->kind == svn_opt_revision_committed
+      && (SVN_CLIENT__REVKIND_IS_LOCAL_TO_WC(peg_revision->kind)
           || peg_revision->kind == svn_opt_revision_unspecified))
     {
       int adm_lock_level = SVN_WC__LEVELS_TO_LOCK_FROM_DEPTH(svn_depth_empty);
@@ -1161,9 +1175,9 @@ location_from_path_and_rev(const char **url,
                                      ctx->cancel_func, ctx->cancel_baton,
                                      subpool));
     }
-  SVN_ERR(svn_client__ra_session_from_path(&ra_session, &rev, url, 
+  SVN_ERR(svn_client__ra_session_from_path(&ra_session, &rev, url,
                                            path_or_url, adm_access,
-                                           peg_revision, peg_revision, 
+                                           peg_revision, peg_revision,
                                            ctx, subpool));
   *url = apr_pstrdup(pool, *url);
   *revision = apr_pcalloc(pool, sizeof(**revision));
@@ -1171,11 +1185,11 @@ location_from_path_and_rev(const char **url,
   (*revision)->value.number = rev;
 
   svn_pool_destroy(subpool);
-  
-  if (adm_access)
-    SVN_ERR(svn_wc_adm_close(adm_access));
 
-  return SVN_NO_ERROR;
+  if (adm_access)
+    return svn_wc_adm_close2(adm_access, pool);
+  else
+    return SVN_NO_ERROR;
 }
 
 
@@ -1204,7 +1218,7 @@ svn_client_mergeinfo_log_merged(const char *path_or_url,
   SVN_ERR(location_from_path_and_rev(&merge_source_url, &real_src_peg_revision,
                                      merge_source_path_or_url,
                                      src_peg_revision, ctx, pool));
-    
+
   /* Step 2: We need the union of PATH_OR_URL@PEG_REVISION's mergeinfo
      and MERGE_SOURCE_URL's history.  It's not enough to do path
      matching, because renames in the history of MERGE_SOURCE_URL
@@ -1212,17 +1226,17 @@ svn_client_mergeinfo_log_merged(const char *path_or_url,
      the target, that vastly simplifies matters (we'll have nothing to
      do). */
   /* This get_mergeinfo() call doubles as a mergeinfo capabilities check. */
-  SVN_ERR(get_mergeinfo(&tgt_mergeinfo, &repos_root, path_or_url, 
+  SVN_ERR(get_mergeinfo(&tgt_mergeinfo, &repos_root, path_or_url,
                         peg_revision, ctx, pool));
   if (! tgt_mergeinfo)
     return SVN_NO_ERROR;
-  SVN_ERR(svn_client__get_history_as_mergeinfo(&source_history, 
+  SVN_ERR(svn_client__get_history_as_mergeinfo(&source_history,
                                                merge_source_url,
                                                real_src_peg_revision,
                                                SVN_INVALID_REVNUM,
                                                SVN_INVALID_REVNUM,
                                                NULL, NULL, ctx, pool));
-  SVN_ERR(svn_mergeinfo_intersect(&mergeinfo, tgt_mergeinfo, 
+  SVN_ERR(svn_mergeinfo_intersect(&mergeinfo, tgt_mergeinfo,
                                   source_history, pool));
 
   /* Step 3: Now, we iterate over the eligible paths/rangelists to
@@ -1244,7 +1258,7 @@ svn_client_mergeinfo_log_merged(const char *path_or_url,
       apr_hash_this(hi, &key, NULL, &val);
       list = val;
       range = APR_ARRAY_IDX(list, list->nelts - 1, svn_merge_range_t *);
-      if ((! SVN_IS_VALID_REVNUM(youngest_rev)) 
+      if ((! SVN_IS_VALID_REVNUM(youngest_rev))
           || (range->end > youngest_rev))
         {
           youngest_rev = range->end;
@@ -1260,8 +1274,8 @@ svn_client_mergeinfo_log_merged(const char *path_or_url,
   /* Step 4: Finally, we run 'svn log' to drive our log receiver, but
      using a receiver filter to only allow revisions to pass through
      that are in our rangelist. */
-  log_target = svn_path_url_add_component(repos_root, log_target + 1, pool);
-  return logs_for_mergeinfo_rangelist(log_target, rangelist, 
+  log_target = svn_path_url_add_component2(repos_root, log_target + 1, pool);
+  return logs_for_mergeinfo_rangelist(log_target, rangelist,
                                       discover_changed_paths, revprops,
                                       log_receiver, log_receiver_baton,
                                       ctx, pool);
@@ -1278,8 +1292,8 @@ svn_client_mergeinfo_get_merged(apr_hash_t **mergeinfo_p,
   const char *repos_root;
   apr_hash_t *full_path_mergeinfo;
   svn_mergeinfo_t mergeinfo;
-  
-  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url, 
+
+  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url,
                         peg_revision, ctx, pool));
 
   /* Copy the MERGEINFO hash items into another hash, but change
@@ -1299,7 +1313,7 @@ svn_client_mergeinfo_get_merged(apr_hash_t **mergeinfo_p,
           apr_hash_this(hi, &key, NULL, &val);
           source_url = svn_path_uri_encode(key, pool);
           source_url = svn_path_join(repos_root, source_url + 1, pool);
-          apr_hash_set(full_path_mergeinfo, source_url, 
+          apr_hash_set(full_path_mergeinfo, source_url,
                        APR_HASH_KEY_STRING, val);
         }
       *mergeinfo_p = full_path_mergeinfo;
@@ -1335,14 +1349,14 @@ svn_client_mergeinfo_log_eligible(const char *path_or_url,
   SVN_ERR(location_from_path_and_rev(&merge_source_url, &real_src_peg_revision,
                                      merge_source_path_or_url,
                                      src_peg_revision, ctx, pool));
-  
+
   /* Step 2: Across the set of possible merges, see what's already
      been merged into PATH_OR_URL@PEG_REVISION (or what's already part
      of the history it shares with that of MERGE_SOURCE_URL.  */
   /* This get_mergeinfo() call doubles as a mergeinfo capabilities check. */
-  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url, 
+  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url,
                         peg_revision, ctx, pool));
-  SVN_ERR(svn_client__get_history_as_mergeinfo(&history, 
+  SVN_ERR(svn_client__get_history_as_mergeinfo(&history,
                                                path_or_url,
                                                peg_revision,
                                                SVN_INVALID_REVNUM,
@@ -1359,7 +1373,7 @@ svn_client_mergeinfo_log_eligible(const char *path_or_url,
   SVN_ERR(svn_client__open_ra_session_internal(&ra_session, merge_source_url,
                                                NULL, NULL, NULL, FALSE,
                                                TRUE, ctx, sesspool));
-  SVN_ERR(svn_client__get_history_as_mergeinfo(&source_history, 
+  SVN_ERR(svn_client__get_history_as_mergeinfo(&source_history,
                                                merge_source_url,
                                                real_src_peg_revision,
                                                SVN_INVALID_REVNUM,
@@ -1390,7 +1404,7 @@ svn_client_mergeinfo_log_eligible(const char *path_or_url,
       apr_hash_this(hi, &key, NULL, &val);
       list = val;
       range = APR_ARRAY_IDX(list, list->nelts - 1, svn_merge_range_t *);
-      if ((! SVN_IS_VALID_REVNUM(youngest_rev)) 
+      if ((! SVN_IS_VALID_REVNUM(youngest_rev))
           || (range->end > youngest_rev))
         {
           youngest_rev = range->end;
@@ -1406,8 +1420,8 @@ svn_client_mergeinfo_log_eligible(const char *path_or_url,
   /* Step 5: Finally, we run 'svn log' to drive our log receiver, but
      using a receiver filter to only allow revisions to pass through
      that are in our rangelist. */
-  log_target = svn_path_url_add_component(repos_root, log_target + 1, pool);
-  return logs_for_mergeinfo_rangelist(log_target, rangelist, 
+  log_target = svn_path_url_add_component2(repos_root, log_target + 1, pool);
+  return logs_for_mergeinfo_rangelist(log_target, rangelist,
                                       discover_changed_paths, revprops,
                                       log_receiver, log_receiver_baton,
                                       ctx, pool);
@@ -1447,15 +1461,15 @@ svn_client_suggest_merge_sources(apr_array_header_t **suggestions,
   */
 
   /* ### TODO: Share ra_session batons to improve efficiency? */
-  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url, 
+  SVN_ERR(get_mergeinfo(&mergeinfo, &repos_root, path_or_url,
                         peg_revision, ctx, pool));
   SVN_ERR(svn_client__get_copy_source(path_or_url, peg_revision,
                                       &copyfrom_path, &copyfrom_rev,
                                       ctx, pool));
   if (copyfrom_path)
-    {   
-      APR_ARRAY_PUSH(list, const char *) = 
-        svn_path_url_add_component(repos_root, copyfrom_path + 1, pool);
+    {
+      APR_ARRAY_PUSH(list, const char *) =
+        svn_path_url_add_component2(repos_root, copyfrom_path, pool);
     }
 
   if (mergeinfo)
@@ -1469,7 +1483,7 @@ svn_client_suggest_merge_sources(apr_array_header_t **suggestions,
           rel_path = key;
           if (copyfrom_path == NULL || strcmp(rel_path, copyfrom_path) != 0)
             APR_ARRAY_PUSH(list, const char *) = \
-              svn_path_url_add_component(repos_root, rel_path + 1, pool);
+              svn_path_url_add_component2(repos_root, rel_path + 1, pool);
         }
     }
 
