@@ -2,7 +2,7 @@
  * diff_file.c :  routines for doing diffs on files
  *
  * ====================================================================
- * Copyright (c) 2000-2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2002-2009 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -38,6 +38,7 @@
 #include "svn_path.h"
 #include "svn_ctype.h"
 
+#include "private/svn_utf_private.h"
 
 /* A token, i.e. a line read from a file. */
 typedef struct svn_diff__file_token_t
@@ -131,16 +132,14 @@ datasource_to_index(svn_diff_datasource_e datasource)
  */
 static APR_INLINE svn_error_t *
 read_chunk(apr_file_t *file, const char *path,
-           char *buffer, apr_size_t length,
+           char *buffer, apr_off_t length,
            apr_off_t offset, apr_pool_t *pool)
 {
   /* XXX: The final offset may not be the one we asked for.
    * XXX: Check.
    */
   SVN_ERR(svn_io_file_seek(file, APR_SET, &offset, pool));
-  SVN_ERR(svn_io_file_read_full(file, buffer, length, NULL, pool));
-
-  return SVN_NO_ERROR;
+  return svn_io_file_read_full(file, buffer, (apr_size_t) length, NULL, pool);
 }
 
 
@@ -174,7 +173,8 @@ map_or_read_file(apr_file_t **file,
 #if APR_HAS_MMAP
   if (finfo.size > APR_MMAP_THRESHOLD)
     {
-      rv = apr_mmap_create(mm, *file, 0, finfo.size, APR_MMAP_READ, pool);
+      rv = apr_mmap_create(mm, *file, 0, (apr_size_t) finfo.size,
+                           APR_MMAP_READ, pool);
       if (rv == APR_SUCCESS)
         {
           *buffer = (*mm)->mm;
@@ -188,9 +188,10 @@ map_or_read_file(apr_file_t **file,
 
    if (*buffer == NULL && finfo.size > 0)
     {
-      *buffer = apr_palloc(pool, finfo.size);
+      *buffer = apr_palloc(pool, (apr_size_t) finfo.size);
 
-      SVN_ERR(svn_io_file_read_full(*file, *buffer, finfo.size, NULL, pool));
+      SVN_ERR(svn_io_file_read_full(*file, *buffer, (apr_size_t) finfo.size,
+                                    NULL, pool));
 
       /* Since we have the entire contents of the file we can
        * close it now.
@@ -213,7 +214,7 @@ datasource_open(void *baton, svn_diff_datasource_e datasource)
   svn_diff__file_baton_t *file_baton = baton;
   int idx;
   apr_finfo_t finfo;
-  apr_size_t length;
+  apr_off_t length;
   char *curp;
   char *endp;
 
@@ -231,16 +232,14 @@ datasource_open(void *baton, svn_diff_datasource_e datasource)
   if (length == 0)
     return SVN_NO_ERROR;
 
-  endp = curp = apr_palloc(file_baton->pool, length);
+  endp = curp = apr_palloc(file_baton->pool, (apr_size_t) length);
   endp += length;
 
   file_baton->buffer[idx] = file_baton->curp[idx] = curp;
   file_baton->endp[idx] = endp;
 
-  SVN_ERR(read_chunk(file_baton->file[idx], file_baton->path[idx],
-                     curp, length, 0, file_baton->pool));
-
-  return SVN_NO_ERROR;
+  return read_chunk(file_baton->file[idx], file_baton->path[idx],
+                    curp, length, 0, file_baton->pool);
 }
 
 
@@ -266,7 +265,7 @@ datasource_get_next_token(apr_uint32_t *hash, void **token, void *baton,
   char *endp;
   char *curp;
   char *eol;
-  int last_chunk;
+  apr_off_t last_chunk;
   apr_off_t length;
   apr_uint32_t h = 0;
   /* Did the last chunk end in a CR character? */
@@ -488,7 +487,7 @@ token_compare(void *baton, void *token1, void *token2, int *compare)
       /* Compare two chunks (that could be entire tokens if they both reside
        * in memory).
        */
-      *compare = memcmp(bufp[0], bufp[1], len);
+      *compare = memcmp(bufp[0], bufp[1], (size_t) len);
       if (*compare != 0)
         return SVN_NO_ERROR;
 
@@ -1061,6 +1060,7 @@ output_unified_diff_modified(void *baton,
       if (output_baton->show_c_function)
         {
           int p;
+          const char *invalid_character;
 
           /* Save the extra context for later use.
            * Note that the last byte of the hunk_extra_context array is never
@@ -1077,6 +1077,14 @@ output_unified_diff_modified(void *baton,
             {
               output_baton->hunk_extra_context[--p] = '\0';
             }
+          invalid_character =
+            svn_utf__last_valid(output_baton->hunk_extra_context,
+                                SVN_DIFF__EXTRA_CONTEXT_LENGTH);
+          for (p = invalid_character - output_baton->hunk_extra_context;
+               p < SVN_DIFF__EXTRA_CONTEXT_LENGTH; p++)
+            {
+              output_baton->hunk_extra_context[p] = '\0';
+            }
         }
     }
 
@@ -1127,7 +1135,8 @@ output_unified_default_hdr(const char **header, const char *path,
   apr_time_exp_lt(&exploded_time, file_info.mtime);
 
   apr_strftime(time_buffer, &time_len, sizeof(time_buffer) - 1,
-               "%a %b %e %H:%M:%S %Y", &exploded_time);
+  /* Order of date components can be different in different languages */
+               _("%a %b %e %H:%M:%S %Y"), &exploded_time);
 
   SVN_ERR(svn_utf_cstring_to_utf8(&utf8_timestr, time_buffer, pool));
 
@@ -1257,40 +1266,34 @@ svn_diff_file_output_unified3(svn_stream_t *output_stream,
   return SVN_NO_ERROR;
 }
 
-svn_error_t *
-svn_diff_file_output_unified2(svn_stream_t *output_stream,
-                              svn_diff_t *diff,
-                              const char *original_path,
-                              const char *modified_path,
-                              const char *original_header,
-                              const char *modified_header,
-                              const char *header_encoding,
-                              apr_pool_t *pool)
-{
-  return svn_diff_file_output_unified3(output_stream, diff,
-                                       original_path, modified_path,
-                                       original_header, modified_header,
-                                       header_encoding, NULL, FALSE, pool);
-}
-
-svn_error_t *
-svn_diff_file_output_unified(svn_stream_t *output_stream,
-                             svn_diff_t *diff,
-                             const char *original_path,
-                             const char *modified_path,
-                             const char *original_header,
-                             const char *modified_header,
-                             apr_pool_t *pool)
-{
-  return svn_diff_file_output_unified2(output_stream, diff,
-                                       original_path, modified_path,
-                                       original_header, modified_header,
-                                       SVN_APR_LOCALE_CHARSET, pool);
-}
-
 
 /** Display diff3 **/
 
+/* A stream to remember *leading* context.  Note that this stream does
+   *not* copy the data that it is remembering; it just saves
+   *pointers! */
+typedef struct {
+  svn_stream_t *stream;
+  const char *data[SVN_DIFF__UNIFIED_CONTEXT_SIZE];
+  apr_size_t len[SVN_DIFF__UNIFIED_CONTEXT_SIZE];
+  apr_size_t next_slot;
+  apr_size_t total_written;
+} context_saver_t;
+
+
+static svn_error_t *
+context_saver_stream_write(void *baton,
+                           const char *data,
+                           apr_size_t *len)
+{
+  context_saver_t *cs = baton;
+  cs->data[cs->next_slot] = data;
+  cs->len[cs->next_slot] = *len;
+  cs->next_slot = (cs->next_slot + 1) % SVN_DIFF__UNIFIED_CONTEXT_SIZE;
+  cs->total_written++;
+  return SVN_NO_ERROR;
+}
+
 typedef struct svn_diff3__file_output_baton_t
 {
   svn_stream_t *output_stream;
@@ -1309,12 +1312,100 @@ typedef struct svn_diff3__file_output_baton_t
   const char *conflict_separator;
   const char *conflict_latest;
 
-  svn_boolean_t display_original_in_conflict;
-  svn_boolean_t display_resolved_conflicts;
+  const char *marker_eol;
 
+  svn_diff_conflict_display_style_t conflict_style;
+
+  /* The rest of the fields are for
+     svn_diff_conflict_display_only_conflicts only.  Note that for
+     these batons, OUTPUT_STREAM is either CONTEXT_SAVER->STREAM or
+     (soon after a conflict) a "trailing context stream", never the
+     actual output stream.*/
+  /* The actual output stream. */
+  svn_stream_t *real_output_stream;
+  context_saver_t *context_saver;
+  /* Used to allocate context_saver and trailing context streams, and
+     for some printfs. */
   apr_pool_t *pool;
 } svn_diff3__file_output_baton_t;
 
+static svn_error_t *
+flush_context_saver(context_saver_t *cs,
+                    svn_stream_t *output_stream)
+{
+  int i;
+  for (i = 0; i < SVN_DIFF__UNIFIED_CONTEXT_SIZE; i++)
+    {
+      int slot = (i + cs->next_slot) % SVN_DIFF__UNIFIED_CONTEXT_SIZE;
+      if (cs->data[slot])
+        {
+          apr_size_t len = cs->len[slot];
+          SVN_ERR(svn_stream_write(output_stream, cs->data[slot], &len));
+        }
+    }
+  return SVN_NO_ERROR;
+}
+
+static void
+make_context_saver(svn_diff3__file_output_baton_t *fob)
+{
+  context_saver_t *cs;
+
+  svn_pool_clear(fob->pool);
+  cs = apr_pcalloc(fob->pool, sizeof(*cs));
+  cs->stream = svn_stream_empty(fob->pool);
+  svn_stream_set_baton(cs->stream, cs);
+  svn_stream_set_write(cs->stream, context_saver_stream_write);
+  fob->context_saver = cs;
+  fob->output_stream = cs->stream;
+}
+
+
+/* A stream which prints SVN_DIFF__UNIFIED_CONTEXT_SIZE lines to
+   BATON->REAL_OUTPUT_STREAM, and then changes BATON->OUTPUT_STREAM to
+   a context_saver; used for *trailing* context. */
+
+struct trailing_context_printer {
+  apr_size_t lines_to_print;
+  svn_diff3__file_output_baton_t *fob;
+};
+
+
+
+static svn_error_t *
+trailing_context_printer_write(void *baton,
+                               const char *data,
+                               apr_size_t *len)
+{
+  struct trailing_context_printer *tcp = baton;
+  SVN_ERR_ASSERT(tcp->lines_to_print > 0);
+  SVN_ERR(svn_stream_write(tcp->fob->real_output_stream, data, len));
+  tcp->lines_to_print--;
+  if (tcp->lines_to_print == 0)
+    make_context_saver(tcp->fob);
+  return SVN_NO_ERROR;
+}
+
+
+static void
+make_trailing_context_printer(svn_diff3__file_output_baton_t *btn)
+{
+  struct trailing_context_printer *tcp;
+  svn_stream_t *s;
+
+  svn_pool_clear(btn->pool);
+
+  tcp = apr_pcalloc(btn->pool, sizeof(*tcp));
+  tcp->lines_to_print = SVN_DIFF__UNIFIED_CONTEXT_SIZE;
+  tcp->fob = btn;
+  s = svn_stream_empty(btn->pool);
+  svn_stream_set_baton(s, tcp);
+  svn_stream_set_write(s, trailing_context_printer_write);
+  btn->output_stream = s;
+}
+
+
+
 typedef enum svn_diff3__file_output_type_e
 {
   svn_diff3__file_output_skip,
@@ -1355,6 +1446,8 @@ output_line(svn_diff3__file_output_baton_t *baton,
   if (type != svn_diff3__file_output_skip)
     {
       len = eol - curp;
+      /* Note that the trailing context printer assumes that
+         svn_stream_write is called exactly once per line. */
       SVN_ERR(svn_stream_write(baton->output_stream, curp, &len));
     }
 
@@ -1363,6 +1456,13 @@ output_line(svn_diff3__file_output_baton_t *baton,
   return SVN_NO_ERROR;
 }
 
+static svn_error_t *
+output_marker_eol(svn_diff3__file_output_baton_t *btn)
+{
+  apr_size_t len = strlen(btn->marker_eol);
+  return svn_stream_write(btn->output_stream, btn->marker_eol, &len);
+}
+
 static svn_error_t *
 output_hunk(void *baton, int idx, apr_off_t target_line,
             apr_off_t target_length)
@@ -1427,6 +1527,66 @@ static const svn_diff_output_fns_t svn_diff3__file_output_vtable =
   output_conflict
 };
 
+
+
+static svn_error_t *
+output_conflict_with_context(svn_diff3__file_output_baton_t *btn,
+                             apr_off_t original_start,
+                             apr_off_t original_length,
+                             apr_off_t modified_start,
+                             apr_off_t modified_length,
+                             apr_off_t latest_start,
+                             apr_off_t latest_length)
+{
+  /* Are we currently saving starting context (as opposed to printing
+     trailing context)?  If so, flush it. */
+  if (btn->output_stream == btn->context_saver->stream)
+    {
+      if (btn->context_saver->total_written > SVN_DIFF__UNIFIED_CONTEXT_SIZE)
+        SVN_ERR(svn_stream_printf(btn->real_output_stream, btn->pool, "@@\n"));
+      SVN_ERR(flush_context_saver(btn->context_saver, btn->real_output_stream));
+    }
+
+  /* Print to the real output stream. */
+  btn->output_stream = btn->real_output_stream;
+
+  /* Output the conflict itself. */
+  SVN_ERR(svn_stream_printf(btn->output_stream, btn->pool,
+                            (modified_length == 1
+                             ? "%s (%" APR_OFF_T_FMT ")"
+                             : "%s (%" APR_OFF_T_FMT ",%" APR_OFF_T_FMT ")"),
+                            btn->conflict_modified,
+                            modified_start + 1, modified_length));
+  SVN_ERR(output_marker_eol(btn));
+  SVN_ERR(output_hunk(btn, 1/*modified*/, modified_start, modified_length));
+
+  SVN_ERR(svn_stream_printf(btn->output_stream, btn->pool,
+                            (original_length == 1
+                             ? "%s (%" APR_OFF_T_FMT ")"
+                             : "%s (%" APR_OFF_T_FMT ",%" APR_OFF_T_FMT ")"),
+                            btn->conflict_original,
+                            original_start + 1, original_length));
+  SVN_ERR(output_marker_eol(btn));
+  SVN_ERR(output_hunk(btn, 0/*original*/, original_start, original_length));
+
+  SVN_ERR(svn_stream_printf(btn->output_stream, btn->pool,
+                            "%s%s", btn->conflict_separator, btn->marker_eol));
+  SVN_ERR(output_hunk(btn, 2/*latest*/, latest_start, latest_length));
+  SVN_ERR(svn_stream_printf(btn->output_stream, btn->pool,
+                            (latest_length == 1
+                             ? "%s (%" APR_OFF_T_FMT ")"
+                             : "%s (%" APR_OFF_T_FMT ",%" APR_OFF_T_FMT ")"),
+                            btn->conflict_latest,
+                            latest_start + 1, latest_length));
+  SVN_ERR(output_marker_eol(btn));
+
+  /* Go into print-trailing-context mode instead. */
+  make_trailing_context_printer(btn);
+
+  return SVN_NO_ERROR;
+}
+
+
 static svn_error_t *
 output_conflict(void *baton,
                 apr_off_t original_start, apr_off_t original_length,
@@ -1437,41 +1597,66 @@ output_conflict(void *baton,
   svn_diff3__file_output_baton_t *file_baton = baton;
   apr_size_t len;
 
-  if (diff && file_baton->display_resolved_conflicts)
+  svn_diff_conflict_display_style_t style = file_baton->conflict_style;
+
+  if (style == svn_diff_conflict_display_only_conflicts)
+    return output_conflict_with_context(file_baton,
+                                        original_start, original_length,
+                                        modified_start, modified_length,
+                                        latest_start, latest_length);
+
+  if (style == svn_diff_conflict_display_resolved_modified_latest)
     {
-      return svn_diff_output(diff, baton,
-                             &svn_diff3__file_output_vtable);
+      if (diff)
+        return svn_diff_output(diff, baton,
+                               &svn_diff3__file_output_vtable);
+      else
+        style = svn_diff_conflict_display_modified_latest;
     }
 
-  len = strlen(file_baton->conflict_modified);
-  SVN_ERR(svn_stream_write(file_baton->output_stream,
-                           file_baton->conflict_modified,
-                           &len));
-
-  SVN_ERR(output_hunk(baton, 1, modified_start, modified_length));
-
-  if (file_baton->display_original_in_conflict)
+  if (style == svn_diff_conflict_display_modified_latest ||
+      style == svn_diff_conflict_display_modified_original_latest)
     {
-      len = strlen(file_baton->conflict_original);
+      len = strlen(file_baton->conflict_modified);
       SVN_ERR(svn_stream_write(file_baton->output_stream,
-                               file_baton->conflict_original, &len));
+                               file_baton->conflict_modified,
+                               &len));
+      SVN_ERR(output_marker_eol(file_baton));
 
-      SVN_ERR(output_hunk(baton, 0, original_start, original_length));
+      SVN_ERR(output_hunk(baton, 1, modified_start, modified_length));
+
+      if (style == svn_diff_conflict_display_modified_original_latest)
+        {
+          len = strlen(file_baton->conflict_original);
+          SVN_ERR(svn_stream_write(file_baton->output_stream,
+                                   file_baton->conflict_original, &len));
+          SVN_ERR(output_marker_eol(file_baton));
+          SVN_ERR(output_hunk(baton, 0, original_start, original_length));
+        }
+
+      len = strlen(file_baton->conflict_separator);
+      SVN_ERR(svn_stream_write(file_baton->output_stream,
+                               file_baton->conflict_separator, &len));
+      SVN_ERR(output_marker_eol(file_baton));
+
+      SVN_ERR(output_hunk(baton, 2, latest_start, latest_length));
+
+      len = strlen(file_baton->conflict_latest);
+      SVN_ERR(svn_stream_write(file_baton->output_stream,
+                               file_baton->conflict_latest, &len));
+      SVN_ERR(output_marker_eol(file_baton));
     }
-
-  len = strlen(file_baton->conflict_separator);
-  SVN_ERR(svn_stream_write(file_baton->output_stream,
-                           file_baton->conflict_separator, &len));
-
-  SVN_ERR(output_hunk(baton, 2, latest_start, latest_length));
-
-  len = strlen(file_baton->conflict_latest);
-  SVN_ERR(svn_stream_write(file_baton->output_stream,
-                           file_baton->conflict_latest, &len));
+  else if (style == svn_diff_conflict_display_modified)
+    SVN_ERR(output_hunk(baton, 1, modified_start, modified_length));
+  else if (style == svn_diff_conflict_display_latest)
+    SVN_ERR(output_hunk(baton, 2, latest_start, latest_length));
+  else /* unknown style */
+    SVN_ERR_MALFUNCTION();
 
   return SVN_NO_ERROR;
 }
 
+
 /* Return the first eol marker found in [BUF, ENDP) as a
  * NUL-terminated string, or NULL if no eol marker is found.
  *
@@ -1501,18 +1686,17 @@ detect_eol(char *buf, char *endp)
 }
 
 svn_error_t *
-svn_diff_file_output_merge(svn_stream_t *output_stream,
-                           svn_diff_t *diff,
-                           const char *original_path,
-                           const char *modified_path,
-                           const char *latest_path,
-                           const char *conflict_original,
-                           const char *conflict_modified,
-                           const char *conflict_latest,
-                           const char *conflict_separator,
-                           svn_boolean_t display_original_in_conflict,
-                           svn_boolean_t display_resolved_conflicts,
-                           apr_pool_t *pool)
+svn_diff_file_output_merge2(svn_stream_t *output_stream,
+                            svn_diff_t *diff,
+                            const char *original_path,
+                            const char *modified_path,
+                            const char *latest_path,
+                            const char *conflict_original,
+                            const char *conflict_modified,
+                            const char *conflict_latest,
+                            const char *conflict_separator,
+                            svn_diff_conflict_display_style_t style,
+                            apr_pool_t *pool)
 {
   svn_diff3__file_output_baton_t baton;
   apr_file_t *file[3];
@@ -1522,10 +1706,18 @@ svn_diff_file_output_merge(svn_stream_t *output_stream,
   apr_mmap_t *mm[3] = { 0 };
 #endif /* APR_HAS_MMAP */
   const char *eol;
+  svn_boolean_t conflicts_only =
+    (style == svn_diff_conflict_display_only_conflicts);
 
   memset(&baton, 0, sizeof(baton));
-  baton.output_stream = output_stream;
-  baton.pool = pool;
+  if (conflicts_only)
+    {
+      baton.pool = svn_pool_create(pool);
+      make_context_saver(&baton);
+      baton.real_output_stream = output_stream;
+    }
+  else
+    baton.output_stream = output_stream;
   baton.path[0] = original_path;
   baton.path[1] = modified_path;
   baton.path[2] = latest_path;
@@ -1548,9 +1740,7 @@ svn_diff_file_output_merge(svn_stream_t *output_stream,
                                                    latest_path),
                                     pool));
 
-  baton.display_original_in_conflict = display_original_in_conflict;
-  baton.display_resolved_conflicts = display_resolved_conflicts &&
-                                     !display_original_in_conflict;
+  baton.conflict_style = style;
 
   for (idx = 0; idx < 3; idx++)
     {
@@ -1572,16 +1762,7 @@ svn_diff_file_output_merge(svn_stream_t *output_stream,
   eol = detect_eol(baton.buffer[1], baton.endp[1]);
   if (! eol)
     eol = APR_EOL_STR;
-
-  /* Extend our conflict markers with the correct eol marker. */
-  baton.conflict_modified = apr_pstrcat(pool, baton.conflict_modified, eol,
-                                        NULL);
-  baton.conflict_original = apr_pstrcat(pool, baton.conflict_original, eol,
-                                        NULL);
-  baton.conflict_separator = apr_pstrcat(pool, baton.conflict_separator, eol,
-                                         NULL);
-  baton.conflict_latest = apr_pstrcat(pool, baton.conflict_latest, eol,
-                                      NULL);
+  baton.marker_eol = eol;
 
   SVN_ERR(svn_diff_output(diff, &baton,
                           &svn_diff3__file_output_vtable));
@@ -1606,5 +1787,45 @@ svn_diff_file_output_merge(svn_stream_t *output_stream,
         }
     }
 
+  if (conflicts_only)
+    svn_pool_destroy(baton.pool);
+
   return SVN_NO_ERROR;
 }
+
+
+svn_error_t *
+svn_diff_file_output_merge(svn_stream_t *output_stream,
+                           svn_diff_t *diff,
+                           const char *original_path,
+                           const char *modified_path,
+                           const char *latest_path,
+                           const char *conflict_original,
+                           const char *conflict_modified,
+                           const char *conflict_latest,
+                           const char *conflict_separator,
+                           svn_boolean_t display_original_in_conflict,
+                           svn_boolean_t display_resolved_conflicts,
+                           apr_pool_t *pool)
+{
+  svn_diff_conflict_display_style_t style =
+    svn_diff_conflict_display_modified_latest;
+
+  if (display_resolved_conflicts)
+    style = svn_diff_conflict_display_resolved_modified_latest;
+
+  if (display_original_in_conflict)
+    style = svn_diff_conflict_display_modified_original_latest;
+
+  return svn_diff_file_output_merge2(output_stream,
+                                     diff,
+                                     original_path,
+                                     modified_path,
+                                     latest_path,
+                                     conflict_original,
+                                     conflict_modified,
+                                     conflict_latest,
+                                     conflict_separator,
+                                     style,
+                                     pool);
+}
