@@ -1,7 +1,7 @@
 /* fs.c --- creating, opening and closing filesystems
  *
  * ====================================================================
- * Copyright (c) 2000-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2007, 2009 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -19,9 +19,6 @@
 #include <stdio.h>
 #include <string.h>
 
-#define APU_WANT_DB
-#include <apu_want.h>
-
 #include <apr_general.h>
 #include <apr_pools.h>
 #include <apr_file_io.h>
@@ -40,6 +37,7 @@
 #include "tree.h"
 #include "id.h"
 #include "lock.h"
+#define SVN_WANT_BDB
 #include "svn_private_config.h"
 
 #include "bdb/bdb-err.h"
@@ -56,6 +54,8 @@
 #include "bdb/locks-table.h"
 #include "bdb/lock-tokens-table.h"
 #include "bdb/node-origins-table.h"
+#include "bdb/miscellaneous-table.h"
+#include "bdb/checksum-reps-table.h"
 
 #include "../libsvn_fs/fs-loader.h"
 #include "private/svn_fs_util.h"
@@ -169,6 +169,8 @@ cleanup_fs(svn_fs_t *fs)
   SVN_ERR(cleanup_fs_db(fs, &bfd->locks, "locks"));
   SVN_ERR(cleanup_fs_db(fs, &bfd->lock_tokens, "lock-tokens"));
   SVN_ERR(cleanup_fs_db(fs, &bfd->node_origins, "node-origins"));
+  SVN_ERR(cleanup_fs_db(fs, &bfd->checksum_reps, "checksum-reps"));
+  SVN_ERR(cleanup_fs_db(fs, &bfd->miscellaneous, "miscellaneous"));
 
   /* Finally, close the environment.  */
   bfd->bdb = 0;
@@ -454,9 +456,7 @@ bdb_write_config(svn_fs_t *fs)
                                      NULL, fs->pool));
     }
 
-  SVN_ERR(svn_io_file_close(dbconfig_file, fs->pool));
-
-  return SVN_NO_ERROR;
+  return svn_io_file_close(dbconfig_file, fs->pool);
 }
 
 
@@ -499,10 +499,10 @@ static fs_vtable_t fs_vtable = {
 /* Depending on CREATE, create or open the environment and databases
    for filesystem FS in PATH. Use POOL for temporary allocations. */
 static svn_error_t *
-open_databases(svn_fs_t *fs, 
+open_databases(svn_fs_t *fs,
                svn_boolean_t create,
                int format,
-               const char *path, 
+               const char *path,
                apr_pool_t *pool)
 {
   base_fs_data_t *bfd;
@@ -620,6 +620,26 @@ open_databases(svn_fs_t *fs,
                                                            create)));
     }
 
+  if (format >= SVN_FS_BASE__MIN_MISCELLANY_FORMAT)
+    {
+      SVN_ERR(BDB_WRAP(fs, (create
+                            ? "creating 'miscellaneous' table"
+                            : "opening 'miscellaneous' table"),
+                       svn_fs_bdb__open_miscellaneous_table(&bfd->miscellaneous,
+                                                            bfd->bdb->env,
+                                                            create)));
+    }
+
+  if (format >= SVN_FS_BASE__MIN_REP_SHARING_FORMAT)
+    {
+      SVN_ERR(BDB_WRAP(fs, (create
+                            ? "creating 'checksum-reps' table"
+                            : "opening 'checksum-reps' table"),
+                       svn_fs_bdb__open_checksum_reps_table(&bfd->checksum_reps,
+                                                            bfd->bdb->env,
+                                                            create)));
+    }
+
   return SVN_NO_ERROR;
 }
 
@@ -631,6 +651,11 @@ base_create(svn_fs_t *fs, const char *path, apr_pool_t *pool,
   int format = SVN_FS_BASE__FORMAT_NUMBER;
   svn_error_t *svn_err;
 
+  /* See if we had an explicitly specified pre-1.5-compatible.  */
+  if (fs->config && apr_hash_get(fs->config, SVN_FS_CONFIG_PRE_1_6_COMPATIBLE,
+                                 APR_HASH_KEY_STRING))
+    format = 3;
+
   /* See if we had an explicitly specified pre-1.5-compatible.  */
   if (fs->config && apr_hash_get(fs->config, SVN_FS_CONFIG_PRE_1_5_COMPATIBLE,
                                  APR_HASH_KEY_STRING))
@@ -666,8 +691,8 @@ error:
 /* Gaining access to an existing Berkeley DB-based filesystem.  */
 
 svn_error_t *
-svn_fs_base__test_required_feature_format(svn_fs_t *fs, 
-                                          const char *feature, 
+svn_fs_base__test_required_feature_format(svn_fs_t *fs,
+                                          const char *feature,
                                           int requires)
 {
   base_fs_data_t *bfd = fs->fsap_data;
@@ -675,7 +700,7 @@ svn_fs_base__test_required_feature_format(svn_fs_t *fs,
     return svn_error_createf
       (SVN_ERR_UNSUPPORTED_FEATURE, NULL,
        _("The '%s' feature requires version %d of the filesystem schema; "
-         "filesystem '%s' uses only version %d"), 
+         "filesystem '%s' uses only version %d"),
        feature, requires, fs->path, bfd->format);
   return SVN_NO_ERROR;
 }
@@ -686,21 +711,15 @@ svn_fs_base__test_required_feature_format(svn_fs_t *fs,
 static svn_error_t *
 check_format(int format)
 {
-  /* We support format 1, 2 and 3 simultaneously.  */
-  if (format == 1 && SVN_FS_BASE__FORMAT_NUMBER == 2)
-    return SVN_NO_ERROR;
-  if ((format == 1 || format == 2) && SVN_FS_BASE__FORMAT_NUMBER == 3)
+  /* We currently support any format less than the compiled format number
+     simultaneously.  */
+  if (format <= SVN_FS_BASE__FORMAT_NUMBER)
     return SVN_NO_ERROR;
 
-  if (format != SVN_FS_BASE__FORMAT_NUMBER)
-    {
-      return svn_error_createf
-        (SVN_ERR_FS_UNSUPPORTED_FORMAT, NULL,
-         _("Expected FS format '%d'; found format '%d'"),
-         SVN_FS_BASE__FORMAT_NUMBER, format);
-    }
-
-  return SVN_NO_ERROR;
+  return svn_error_createf(
+        SVN_ERR_FS_UNSUPPORTED_FORMAT, NULL,
+        _("Expected FS format '%d'; found format '%d'"),
+        SVN_FS_BASE__FORMAT_NUMBER, format);
 }
 
 static svn_error_t *
@@ -712,8 +731,8 @@ base_open(svn_fs_t *fs, const char *path, apr_pool_t *pool,
   svn_boolean_t write_format_file = FALSE;
 
   /* Read the FS format number. */
-  svn_err = svn_io_read_version_file(&format, 
-                                     svn_path_join(path, FORMAT_FILE, pool), 
+  svn_err = svn_io_read_version_file(&format,
+                                     svn_path_join(path, FORMAT_FILE, pool),
                                      pool);
   if (svn_err && APR_STATUS_IS_ENOENT(svn_err->apr_err))
     {
@@ -730,7 +749,7 @@ base_open(svn_fs_t *fs, const char *path, apr_pool_t *pool,
     goto error;
 
   /* Create the environment and databases. */
-  svn_err = open_databases(fs, FALSE, format, path, pool);  
+  svn_err = open_databases(fs, FALSE, format, path, pool);
   if (svn_err) goto error;
 
   ((base_fs_data_t *) fs->fsap_data)->format = format;
@@ -739,7 +758,7 @@ base_open(svn_fs_t *fs, const char *path, apr_pool_t *pool,
   /* If we lack a format file, write one. */
   if (write_format_file)
     {
-      svn_err = svn_io_write_version_file(svn_path_join(path, FORMAT_FILE, 
+      svn_err = svn_io_write_version_file(svn_path_join(path, FORMAT_FILE,
                                                         pool), format, pool);
       if (svn_err) goto error;
     }
@@ -781,9 +800,7 @@ bdb_recover(const char *path, svn_boolean_t fatal, apr_pool_t *pool)
                            ((fatal ? DB_RECOVER_FATAL : DB_RECOVER)
                             | SVN_BDB_PRIVATE_ENV_FLAGS),
                            0666, pool));
-  SVN_ERR(svn_fs_bdb__close(bdb));
-
-  return SVN_NO_ERROR;
+  return svn_fs_bdb__close(bdb);
 }
 
 static svn_error_t *
@@ -800,10 +817,46 @@ static svn_error_t *
 base_upgrade(svn_fs_t *fs, const char *path, apr_pool_t *pool,
              apr_pool_t *common_pool)
 {
-  /* Currently, upgrading just means bumping the format file's stored
-     version number. */
-  return svn_io_write_version_file(svn_path_join(path, FORMAT_FILE, pool), 
-                                   SVN_FS_BASE__FORMAT_NUMBER, pool);
+  const char *version_file_path;
+  int old_format_number;
+
+  version_file_path = svn_path_join(path, FORMAT_FILE, pool);
+
+  /* Read the old number so we've got it on hand later on. */
+  SVN_ERR(svn_io_read_version_file(&old_format_number, version_file_path,
+                                   pool));
+
+  /* Bump the format file's stored version number. */
+  SVN_ERR(svn_io_write_version_file(version_file_path,
+                                    SVN_FS_BASE__FORMAT_NUMBER, pool));
+
+  /* Check and see if we need to record the "bump" revision. */
+  if (old_format_number < SVN_FS_BASE__MIN_FORWARD_DELTAS_FORMAT)
+    {
+      apr_pool_t *subpool = svn_pool_create(pool);
+      svn_revnum_t youngest_rev;
+      const char *value;
+
+      /* Open the filesystem in a subpool (so we can control its
+         closure) and do our fiddling.
+
+         NOTE: By using base_open() here instead of open_databases(),
+         we will end up re-reading the format file that we just wrote.
+         But it's better to use the existing encapsulation of "opening
+         the filesystem" rather than duplicating (or worse, partially
+         duplicating) that logic here.  */
+      SVN_ERR(base_open(fs, path, subpool, common_pool));
+
+      /* Fetch the youngest rev, and record it */
+      SVN_ERR(svn_fs_base__youngest_rev(&youngest_rev, fs, subpool));
+      value = apr_psprintf(subpool, "%ld", youngest_rev);
+      SVN_ERR(svn_fs_base__miscellaneous_set
+              (fs, SVN_FS_BASE__MISC_FORWARD_DELTA_UPGRADE,
+               value, subpool));
+      svn_pool_destroy(subpool);
+    }
+
+  return SVN_NO_ERROR;
 }
 
 static svn_error_t *
@@ -816,6 +869,19 @@ base_bdb_recover(svn_fs_t *fs,
   return bdb_recover(fs->path, FALSE, pool);
 }
 
+static svn_error_t *
+base_bdb_pack(svn_fs_t *fs,
+              const char *path,
+              svn_fs_pack_notify_t notify_func,
+              void *notify_baton,
+              svn_cancel_func_t cancel,
+              void *cancel_baton,
+              apr_pool_t *pool)
+{
+  /* Packing is currently a no op for BDB. */
+  return SVN_NO_ERROR;
+}
+
 
 
 /* Running the 'archive' command on a Berkeley DB-based filesystem.  */
@@ -904,7 +970,7 @@ svn_fs_base__clean_logs(const char *live_path,
                                                  backup_log_path,
                                                  sub_pool));
 
-          /* If log files do not match, go to the next log filr. */
+          /* If log files do not match, go to the next log file. */
           if (files_match == FALSE)
             continue;
         }
@@ -1156,6 +1222,10 @@ base_hotcopy(const char *src_path,
                               "lock-tokens", pagesize, TRUE, pool));
   SVN_ERR(copy_db_file_safely(src_path, dest_path,
                               "node-origins", pagesize, TRUE, pool));
+  SVN_ERR(copy_db_file_safely(src_path, dest_path,
+                              "checksum-reps", pagesize, TRUE, pool));
+  SVN_ERR(copy_db_file_safely(src_path, dest_path,
+                              "miscellaneous", pagesize, TRUE, pool));
 
   {
     apr_array_header_t *logfiles;
@@ -1234,9 +1304,7 @@ base_delete_fs(const char *path,
   SVN_ERR(svn_fs_bdb__remove(path, pool));
 
   /* Remove the environment directory. */
-  SVN_ERR(svn_io_remove_dir2(path, FALSE, NULL, NULL, pool));
-
-  return SVN_NO_ERROR;
+  return svn_io_remove_dir2(path, FALSE, NULL, NULL, pool);
 }
 
 static const svn_version_t *
@@ -1264,6 +1332,7 @@ static fs_library_vtable_t library_vtable = {
   base_hotcopy,
   base_get_description,
   base_bdb_recover,
+  base_bdb_pack,
   base_bdb_logfiles,
   svn_fs_base__id_parse
 };
