@@ -2,7 +2,7 @@
  * questions.c:  routines for asking questions about working copies
  *
  * ====================================================================
- * Copyright (c) 2000-2004, 2006 CollabNet.  All rights reserved.
+ * Copyright (c) 2000-2004, 2006, 2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -21,6 +21,7 @@
 #include <string.h>
 #include <apr_pools.h>
 #include <apr_file_io.h>
+#include <apr_file_info.h>
 #include <apr_time.h>
 #include "svn_pools.h"
 #include "svn_types.h"
@@ -28,7 +29,6 @@
 #include "svn_error.h"
 #include "svn_path.h"
 #include "svn_time.h"
-#include "svn_wc.h"
 #include "svn_io.h"
 #include "svn_props.h"
 
@@ -36,26 +36,23 @@
 #include "adm_files.h"
 #include "questions.h"
 #include "entries.h"
+#include "props.h"
 #include "translate.h"
 
-#include "svn_md5.h"
-#include <apr_md5.h>
-
 #include "svn_private_config.h"
+#include "private/svn_wc_private.h"
 
 
 /* ### todo: make this compare repository too?  Or do so in parallel
-   code.  See also adm_files.c:check_adm_exists(), which should
-   probably be merged with this.  */
+   code.  */
 svn_error_t *
 svn_wc_check_wc(const char *path,
                 int *wc_format,
                 apr_pool_t *pool)
 {
-  svn_error_t *err = SVN_NO_ERROR;
-
-  const char *format_file_path
-    = svn_wc__adm_path(path, FALSE, pool, SVN_WC__ADM_ENTRIES, NULL);
+  svn_error_t *err;
+  const char *format_file_path = svn_wc__adm_child(path, SVN_WC__ADM_ENTRIES,
+                                                   pool);
 
   /* First try to read the format number from the entries file. */
   err = svn_io_read_version_file(wc_format, format_file_path, pool);
@@ -69,11 +66,10 @@ svn_wc_check_wc(const char *path,
          Note that the format file might not exist in newer working copies
          (format 7 and higher), but in that case, the entries file should
          have contained the format number. */
-      format_file_path
-        = svn_wc__adm_path(path, FALSE, pool, SVN_WC__ADM_FORMAT, NULL);
+      format_file_path = svn_wc__adm_child(path, SVN_WC__ADM_FORMAT, pool);
 
       err = svn_io_read_version_file(wc_format, format_file_path, pool);
-    }      
+    }
 
   if (err && (APR_STATUS_IS_ENOENT(err->apr_err)
               || APR_STATUS_IS_ENOTDIR(err->apr_err)))
@@ -123,10 +119,18 @@ svn_wc__check_format(int wc_format, const char *path, apr_pool_t *pool)
     }
   else if (wc_format > SVN_WC__VERSION)
     {
+      /* This won't do us much good for the 1.4<->1.5 crossgrade,
+         since 1.4.x clients don't refer to this FAQ entry, but at
+         least post-1.5 crossgrades will be somewhat less painful. */
       return svn_error_createf
         (SVN_ERR_WC_UNSUPPORTED_FORMAT, NULL,
-         _("This client is too old to work with working copy '%s'; "
-           "please get a newer Subversion client"),
+         _("This client is too old to work with working copy '%s'.  You need\n"
+           "to get a newer Subversion client, or to downgrade this working "
+           "copy.\n"
+           "See "
+           "http://subversion.tigris.org/faq.html#working-copy-format-change\n"
+           "for details."
+           ),
          svn_path_local_style(path, pool));
     }
 
@@ -140,16 +144,20 @@ svn_wc__check_format(int wc_format, const char *path, apr_pool_t *pool)
 /* svn_wc_text_modified_p answers the question:
 
    "Are the contents of F different than the contents of
-   .svn/text-base/F.svn-base?"
+   .svn/text-base/F.svn-base or .svn/tmp/text-base/F.svn-base?"
 
-   In other words, we're looking to see if a user has made local
-   modifications to a file since the last update or commit.
+   In the first case, we're looking to see if a user has made local
+   modifications to a file since the last update or commit.  In the
+   second, the file may not be versioned yet (it doesn't exist in
+   entries).  Support for the latter case came about to facilitate
+   forced checkouts, updates, and switches, where an unversioned file
+   may obstruct a file about to be added.
 
    Note: Assuming that F lives in a directory D at revision V, please
    notice that we are *NOT* answering the question, "are the contents
    of F different than revision V of F?"  While F may be at a different
    revision number than its parent directory, but we're only looking
-   for local edits on F, not for consistent directory revisions.  
+   for local edits on F, not for consistent directory revisions.
 
    TODO:  the logic of the routines on this page might change in the
    future, as they bear some relation to the user interface.  For
@@ -159,6 +167,7 @@ svn_wc__check_format(int wc_format, const char *path, apr_pool_t *pool)
    mark it for removal?
 */
 
+
 /* Is PATH's timestamp the same as the one recorded in our
    `entries' file?  Return the answer in EQUAL_P.  TIMESTAMP_KIND
    should be one of the enumerated type above. */
@@ -166,65 +175,18 @@ svn_error_t *
 svn_wc__timestamps_equal_p(svn_boolean_t *equal_p,
                            const char *path,
                            svn_wc_adm_access_t *adm_access,
-                           enum svn_wc__timestamp_kind timestamp_kind,
                            apr_pool_t *pool)
 {
-  apr_time_t wfile_time, entrytime = 0;
   const svn_wc_entry_t *entry;
+  apr_time_t wfile_time;
 
   /* Get the timestamp from the entries file */
-  SVN_ERR(svn_wc_entry(&entry, path, adm_access, FALSE, pool));
-
-  /* Can't compare timestamps for an unversioned file. */
-  if (entry == NULL)
-    return svn_error_createf
-      (SVN_ERR_ENTRY_NOT_FOUND, NULL,
-       _("'%s' is not under version control"),
-       svn_path_local_style(path, pool));
+  SVN_ERR(svn_wc__entry_versioned(&entry, path, adm_access, FALSE, pool));
 
   /* Get the timestamp from the working file and the entry */
-  if (timestamp_kind == svn_wc__text_time)
-    {
-      SVN_ERR(svn_io_file_affected_time(&wfile_time, path, pool));
-      entrytime = entry->text_time;
-    }
-  
-  else if (timestamp_kind == svn_wc__prop_time)
-    {
-      const char *prop_path;
+  SVN_ERR(svn_io_file_affected_time(&wfile_time, path, pool));
 
-      SVN_ERR(svn_wc__prop_path(&prop_path, path, entry->kind, FALSE, pool));
-      SVN_ERR(svn_io_file_affected_time(&wfile_time, prop_path, pool));
-      entrytime = entry->prop_time;
-    }
-
-  if (! entrytime)
-    {
-      /* TODO: If either timestamp is inaccessible, the test cannot
-         return an answer.  Assume that the timestamps are
-         different. */
-      *equal_p = FALSE;
-      return SVN_NO_ERROR;
-    }
-
-  {
-    /* Put the disk timestamp through a string conversion, so it's
-       at the same resolution as entry timestamps. */
-    /* This string conversion here may be goodness, but it does
-       nothing currently _and_ it is somewhat expensive _and_ it eats
-       memory _and_ it is tested for in the regression tests. But I
-       will only comment it out because I do not possess the guts to
-       remove it altogether. */
-    /*
-    const char *tstr = svn_time_to_cstring (wfile_time, pool);
-    SVN_ERR (svn_time_from_cstring (&wfile_time, tstr, pool));
-    */
-  }
-  
-  if (wfile_time == entrytime)
-    *equal_p = TRUE;
-  else
-    *equal_p = FALSE;
+  *equal_p = wfile_time == entry->text_time;
 
   return SVN_NO_ERROR;
 }
@@ -262,59 +224,60 @@ compare_and_verify(svn_boolean_t *modified_p,
                               adm_access, NULL, pool));
   SVN_ERR(svn_wc__get_special(&special, versioned_file, adm_access, pool));
 
-
   need_translation = svn_subst_translation_required(eol_style, eol_str,
                                                     keywords, special, TRUE);
+
   if (verify_checksum || need_translation)
     {
       /* Reading files is necessary. */
-      const unsigned char *digest;
-      /* "v_" means versioned_file, "b_" means base_file. */
-      apr_file_t *v_file_h, *b_file_h;
-      svn_stream_t *v_stream, *b_stream;
+      svn_checksum_t *checksum;
+      svn_stream_t *v_stream;  /* versioned_file */
+      svn_stream_t *b_stream;  /* base_file */
       const svn_wc_entry_t *entry;
-      
-      SVN_ERR(svn_io_file_open(&b_file_h, base_file, APR_READ,
-                              APR_OS_DEFAULT, pool));
 
-      b_stream = svn_stream_from_aprfile2(b_file_h, FALSE, pool);
+      SVN_ERR(svn_stream_open_readonly(&b_stream, base_file, pool, pool));
 
       if (verify_checksum)
         {
           /* Need checksum verification, so read checksum from entries file
            * and setup checksummed stream for base file. */
-          SVN_ERR(svn_wc_entry(&entry, versioned_file, adm_access, TRUE,
-                               pool));
-          if (! entry)
-            return svn_error_createf
-              (SVN_ERR_UNVERSIONED_RESOURCE, NULL,
-                _("'%s' is not under version control"),
-                svn_path_local_style(versioned_file, pool));
+          SVN_ERR(svn_wc__entry_versioned(&entry, versioned_file, adm_access,
+                                         TRUE, pool));
 
           if (entry->checksum)
-            b_stream = svn_stream_checksummed(b_stream, &digest, NULL, TRUE,
-                                              pool);
+            b_stream = svn_stream_checksummed2(b_stream, &checksum, NULL,
+                                               svn_checksum_md5, TRUE, pool);
         }
 
-      if (compare_textbases && need_translation)
+      if (special)
         {
-          /* Create stream for detranslate versioned file to normal form. */
-          SVN_ERR(svn_subst_stream_detranslated(&v_stream,
-                                                versioned_file,
-                                                eol_style,
-                                                eol_str, TRUE,
-                                                keywords, special,
-                                                pool));
+          SVN_ERR(svn_subst_read_specialfile(&v_stream, versioned_file,
+                                             pool, pool));
         }
       else
         {
-          SVN_ERR(svn_io_file_open(&v_file_h, versioned_file, APR_READ,
-                              APR_OS_DEFAULT, pool));
-          v_stream = svn_stream_from_aprfile2(v_file_h, FALSE, pool);
+          SVN_ERR(svn_stream_open_readonly(&v_stream, versioned_file,
+                                           pool, pool));
 
-          if (need_translation)
+          if (compare_textbases && need_translation)
             {
-              /* Translate text-base to working copy form. */
+              if (eol_style == svn_subst_eol_style_native)
+                eol_str = SVN_SUBST_NATIVE_EOL_STR;
+              else if (eol_style != svn_subst_eol_style_fixed
+                       && eol_style != svn_subst_eol_style_none)
+                return svn_error_create(SVN_ERR_IO_UNKNOWN_EOL, NULL, NULL);
+
+              /* Wrap file stream to detranslate into normal form. */
+              v_stream = svn_subst_stream_translated(v_stream,
+                                                     eol_str,
+                                                     TRUE,
+                                                     keywords,
+                                                     FALSE /* expand */,
+                                                     pool);
+            }
+          else if (need_translation)
+            {
+              /* Wrap base stream to translate into working copy form. */
               b_stream = svn_subst_stream_translated(b_stream, eol_str,
                                                      FALSE, keywords, TRUE,
                                                      pool);
@@ -322,15 +285,15 @@ compare_and_verify(svn_boolean_t *modified_p,
         }
 
       SVN_ERR(svn_stream_contents_same(&same, b_stream, v_stream, pool));
-      
+
       SVN_ERR(svn_stream_close(v_stream));
       SVN_ERR(svn_stream_close(b_stream));
 
       if (verify_checksum && entry->checksum)
         {
-          const char *checksum;
-          checksum = svn_md5_digest_to_cstring_display(digest, pool);
-          if (strcmp(checksum, entry->checksum) != 0)
+          const char *digest;
+          digest = svn_checksum_to_cstring_display(checksum, pool);
+          if (strcmp(digest, entry->checksum) != 0)
             {
               return svn_error_createf
                 (SVN_ERR_WC_CORRUPT_TEXT_BASE, NULL,
@@ -339,7 +302,7 @@ compare_and_verify(svn_boolean_t *modified_p,
                     "     actual:  %s\n"),
                   svn_path_local_style(base_file, pool),
                   entry->checksum,
-                  checksum);
+                  digest);
             }
         }
     }
@@ -350,9 +313,8 @@ compare_and_verify(svn_boolean_t *modified_p,
                                            pool));
     }
 
-
-
   *modified_p = (! same);
+
   return SVN_NO_ERROR;
 }
 
@@ -377,97 +339,160 @@ svn_wc__text_modified_internal_p(svn_boolean_t *modified_p,
                                  apr_pool_t *pool)
 {
   const char *textbase_filename;
-  svn_boolean_t equal_timestamps;
-  apr_pool_t *subpool = svn_pool_create(pool);
   svn_node_kind_t kind;
   svn_error_t *err;
+  apr_finfo_t finfo;
+
+
+  /* No matter which way you look at it, the file needs to exist. */
+  err = svn_io_stat(&finfo, filename,
+                    APR_FINFO_SIZE | APR_FINFO_MTIME | APR_FINFO_TYPE
+                    | APR_FINFO_LINK, pool);
+  if ((err && APR_STATUS_IS_ENOENT(err->apr_err))
+      || (!err && !(finfo.filetype == APR_REG ||
+                    finfo.filetype == APR_LNK)))
+    {
+      /* There is no entity, or, the entity is not a regular file or link.
+         So, it can't be modified. */
+      svn_error_clear(err);
+      *modified_p = FALSE;
+      return SVN_NO_ERROR;
+    }
+  else if (err)
+    return err;
 
   if (! force_comparison)
     {
-      /* See if the local file's timestamp is the same as the one
-         recorded in the administrative directory.  This could,
-         theoretically, be wrong in certain rare cases, but with the
-         addition of a forced delay after commits (see revision 419
-         and issue #542) it's highly unlikely to be a problem. */
-      err = svn_wc__timestamps_equal_p(&equal_timestamps,
-                                       filename, adm_access,
-                                       svn_wc__text_time, subpool);
+      const svn_wc_entry_t *entry;
 
-      /* We only care whether there was an error or not, so make sure it
-         is cleared. */
-      svn_error_clear(err);
+      /* We're allowed to use a heuristic to determine whether files may
+         have changed.  The heuristic has these steps:
 
-      /* If we have an error, we fall back on the slower code path below.
-         It might be tempting to optimize this further, for example by
-         detecting when the file didn't exists.  But we have to be careful
-         with what error codes we return.  If the file doesn't exist,
-         we should return no error.  But, *if* it exists, but it is
-         unversioned, we have to return SVN_ERR_ENTRY_NOT_FOUND. */
-      if (! err && equal_timestamps)
+
+         1. Compare the working file's size
+            with the size cached in the entries file
+         2. If they differ, do a full file compare
+         3. Compare the working file's timestamp
+            with the timestamp cached in the entries file
+         4. If they differ, do a full file compare
+         5. Otherwise, return indicating an unchanged file.
+
+         There are 2 problematic situations which may occur:
+
+         1. The cached working size is missing
+         --> In this case, we forget we ever tried to compare
+             and skip to the timestamp comparison.  This is
+             because old working copies do not contain cached sizes
+
+         2. The cached timestamp is missing
+         --> In this case, we forget we ever tried to compare
+             and skip to full file comparison.  This is because
+             the timestamp will be removed when the library
+             updates a locally changed file.  (ie, this only happens
+             when the file was locally modified.)
+
+      */
+
+
+      /* Get the entry */
+      err = svn_wc_entry(&entry, filename, adm_access, FALSE, pool);
+      if (err)
         {
-          *modified_p = FALSE;
-          goto cleanup;
+          svn_error_clear(err);
+          goto compare_them;
         }
-    }
 
-  /* Make sure the file exists before proceeding. */
-  SVN_ERR(svn_io_check_path(filename, &kind, pool));
-  if (kind != svn_node_file)
-    {
-      /* If the file doesn't exist, consider it non-modified. */
+      if (! entry)
+        goto compare_them;
+
+      /* Compare the sizes, if applicable */
+      if (entry->working_size != SVN_WC_ENTRY_WORKING_SIZE_UNKNOWN
+          && finfo.size != entry->working_size)
+        goto compare_them;
+
+
+      /* Compare the timestamps
+
+         Note: text_time == 0 means absent from entries,
+               which also means the timestamps won't be equal,
+               so there's no need to explicitly check the 'absent' value. */
+      if (entry->text_time != finfo.mtime)
+        goto compare_them;
+
+
       *modified_p = FALSE;
-      goto cleanup;
+      return SVN_NO_ERROR;
     }
 
-  /* If there's no text-base file, we have to assume the working file
+ compare_them:
+ /* If there's no text-base file, we have to assume the working file
      is modified.  For example, a file scheduled for addition but not
      yet committed. */
-  textbase_filename = svn_wc__text_base_path(filename, 0, subpool);
-  SVN_ERR(svn_io_check_path(textbase_filename, &kind, subpool));
-  if (kind != svn_node_file)
-    {
-      *modified_p = TRUE;
-      goto cleanup;
-    }
-
+  /* We used to stat for the working base here, but we just give
+     compare_and_verify a try; we'll check for errors afterwards */
+  textbase_filename = svn_wc__text_base_path(filename, FALSE, pool);
 
   /* Check all bytes, and verify checksum if requested. */
-  SVN_ERR(compare_and_verify(modified_p,
+  {
+    apr_pool_t *subpool = svn_pool_create(pool);
+
+    err = compare_and_verify(modified_p,
                              filename,
                              adm_access,
                              textbase_filename,
                              compare_textbases,
                              force_comparison,
-                             subpool));
+                             subpool);
+    if (err)
+      {
+        svn_error_t *err2;
+
+        err2 = svn_io_check_path(textbase_filename, &kind, pool);
+        if (! err2 && kind != svn_node_file)
+          {
+            svn_error_clear(err);
+            *modified_p = TRUE;
+            return SVN_NO_ERROR;
+          }
+
+        svn_error_clear(err);
+        return err2;
+      }
+
+    svn_pool_destroy(subpool);
+  }
 
   /* It is quite legitimate for modifications to the working copy to
      produce a timestamp variation with no text variation. If it turns out
      that there are no differences then we might be able to "repair" the
      text-time in the entries file and so avoid the expensive file contents
-     comparison in the future. */
+     comparison in the future.
+     Though less likely, the same may be true for the size
+     of the working file. */
   if (! *modified_p && svn_wc_adm_locked(adm_access))
     {
       svn_wc_entry_t tmp;
-      SVN_ERR(svn_io_file_affected_time(&tmp.text_time, filename, pool));
+
+      tmp.working_size = finfo.size;
+      tmp.text_time = finfo.mtime;
       SVN_ERR(svn_wc__entry_modify(adm_access,
                                    svn_path_basename(filename, pool),
-                                   &tmp, SVN_WC__ENTRY_MODIFY_TEXT_TIME, TRUE,
-                                   pool));
+                                   &tmp,
+                                   SVN_WC__ENTRY_MODIFY_TEXT_TIME
+                                   | SVN_WC__ENTRY_MODIFY_WORKING_SIZE,
+                                   TRUE, pool));
     }
 
- cleanup:
-  svn_pool_destroy(subpool);
-
   return SVN_NO_ERROR;
 }
 
 
 svn_error_t *
 svn_wc_text_modified_p(svn_boolean_t *modified_p,
-                       const char *filename,
-                       svn_boolean_t force_comparison,
-                       svn_wc_adm_access_t *adm_access,
-                       apr_pool_t *pool)
+                        const char *filename,
+                        svn_boolean_t force_comparison,
+                        svn_wc_adm_access_t *adm_access,
+                        apr_pool_t *pool)
 {
   return svn_wc__text_modified_internal_p(modified_p, filename,
                                           force_comparison, adm_access,
@@ -476,6 +501,87 @@ svn_wc_text_modified_p(svn_boolean_t *modified_p,
 
 
 
+svn_error_t *
+svn_wc_conflicted_p2(svn_boolean_t *text_conflicted_p,
+                     svn_boolean_t *prop_conflicted_p,
+                     svn_boolean_t *tree_conflicted_p,
+                     const char *path,
+                     svn_wc_adm_access_t *adm_access,
+                     apr_pool_t *pool)
+{
+  svn_node_kind_t kind;
+  const svn_wc_entry_t *entry;
+  const char* dir_path = svn_path_dirname(path, pool);
+
+  SVN_ERR(svn_wc_entry(&entry, path, adm_access, TRUE, pool));
+
+  if (text_conflicted_p)
+    {
+      *text_conflicted_p = FALSE;
+
+      if (entry)
+        {
+          /* Look for any text conflict, exercising only as much effort as
+             necessary to obtain a definitive answer.  This only applies to
+             files, but we don't have to explicitly check that entry is a
+             file, since these attributes would never be set on a directory
+             anyway.  A conflict file entry notation only counts if the
+             conflict file still exists on disk.  */
+
+          if (entry->conflict_old)
+            {
+              path = svn_path_join(dir_path, entry->conflict_old, pool);
+              SVN_ERR(svn_io_check_path(path, &kind, pool));
+              *text_conflicted_p = (kind == svn_node_file);
+            }
+
+          if ((! *text_conflicted_p) && (entry->conflict_new))
+            {
+              path = svn_path_join(dir_path, entry->conflict_new, pool);
+              SVN_ERR(svn_io_check_path(path, &kind, pool));
+              *text_conflicted_p = (kind == svn_node_file);
+            }
+
+          if ((! *text_conflicted_p) && (entry->conflict_wrk))
+            {
+              path = svn_path_join(dir_path, entry->conflict_wrk, pool);
+              SVN_ERR(svn_io_check_path(path, &kind, pool));
+              *text_conflicted_p = (kind == svn_node_file);
+            }
+        }
+    }
+
+  /* What about prop conflicts? */
+  if (prop_conflicted_p)
+    {
+      *prop_conflicted_p = FALSE;
+
+      if (entry && entry->prejfile)
+        {
+          /* A dir's .prej file is _inside_ the dir. */
+          if (entry->kind == svn_node_dir)
+            path = svn_path_join(path, entry->prejfile, pool);
+          else
+            path = svn_path_join(dir_path, entry->prejfile, pool);
+
+          SVN_ERR(svn_io_check_path(path, &kind, pool));
+          *prop_conflicted_p = (kind == svn_node_file);
+        }
+    }
+
+  /* Find out whether it's a tree conflict victim. */
+  if (tree_conflicted_p)
+    {
+      svn_wc_conflict_description_t *conflict;
+
+      SVN_ERR_ASSERT(adm_access != NULL);
+      SVN_ERR(svn_wc__get_tree_conflict(&conflict, path, adm_access, pool));
+      *tree_conflicted_p = (conflict != NULL);
+    }
+
+  return SVN_NO_ERROR;
+}
+
 svn_error_t *
 svn_wc_conflicted_p(svn_boolean_t *text_conflicted_p,
                     svn_boolean_t *prop_conflicted_p,
@@ -483,58 +589,43 @@ svn_wc_conflicted_p(svn_boolean_t *text_conflicted_p,
                     const svn_wc_entry_t *entry,
                     apr_pool_t *pool)
 {
-  const char *path;
   svn_node_kind_t kind;
-  apr_pool_t *subpool = svn_pool_create(pool);  /* ### Why? */
+  const char *path;
 
   *text_conflicted_p = FALSE;
   *prop_conflicted_p = FALSE;
 
-  /* Look for any text conflict, exercising only as much effort as
-     necessary to obtain a definitive answer.  This only applies to
-     files, but we don't have to explicitly check that entry is a
-     file, since these attributes would never be set on a directory
-     anyway.  A conflict file entry notation only counts if the
-     conflict file still exists on disk.  */
   if (entry->conflict_old)
     {
-      path = svn_path_join(dir_path, entry->conflict_old, subpool);
-      SVN_ERR(svn_io_check_path(path, &kind, subpool));
-      if (kind == svn_node_file)
-        *text_conflicted_p = TRUE;
+      path = svn_path_join(dir_path, entry->conflict_old, pool);
+      SVN_ERR(svn_io_check_path(path, &kind, pool));
+      *text_conflicted_p = (kind == svn_node_file);
     }
 
   if ((! *text_conflicted_p) && (entry->conflict_new))
     {
-      path = svn_path_join(dir_path, entry->conflict_new, subpool);
-      SVN_ERR(svn_io_check_path(path, &kind, subpool));
-      if (kind == svn_node_file)
-        *text_conflicted_p = TRUE;
+      path = svn_path_join(dir_path, entry->conflict_new, pool);
+      SVN_ERR(svn_io_check_path(path, &kind, pool));
+      *text_conflicted_p = (kind == svn_node_file);
     }
 
   if ((! *text_conflicted_p) && (entry->conflict_wrk))
     {
-      path = svn_path_join(dir_path, entry->conflict_wrk, subpool);
-      SVN_ERR(svn_io_check_path(path, &kind, subpool));
-      if (kind == svn_node_file)
-        *text_conflicted_p = TRUE;
+      path = svn_path_join(dir_path, entry->conflict_wrk, pool);
+      SVN_ERR(svn_io_check_path(path, &kind, pool));
+      *text_conflicted_p = (kind == svn_node_file);
     }
 
-  /* What about prop conflicts? */
   if (entry->prejfile)
     {
-      path = svn_path_join(dir_path, entry->prejfile, subpool);
-      SVN_ERR(svn_io_check_path(path, &kind, subpool));
-      if (kind == svn_node_file)
-        *prop_conflicted_p = TRUE;
+      path = svn_path_join(dir_path, entry->prejfile, pool);
+      SVN_ERR(svn_io_check_path(path, &kind, pool));
+      *prop_conflicted_p = (kind == svn_node_file);
     }
-  
-  svn_pool_destroy(subpool);
+
   return SVN_NO_ERROR;
 }
 
-
-
 
 
 svn_error_t *
@@ -548,12 +639,12 @@ svn_wc_has_binary_prop(svn_boolean_t *has_binary_prop,
 
   SVN_ERR(svn_wc_prop_get(&value, SVN_PROP_MIME_TYPE, path, adm_access,
                           subpool));
- 
+
   if (value && (svn_mime_type_is_binary(value->data)))
     *has_binary_prop = TRUE;
   else
     *has_binary_prop = FALSE;
-  
+
   svn_pool_destroy(subpool);
   return SVN_NO_ERROR;
 }
