@@ -1,6 +1,6 @@
 /*
  * ====================================================================
- * Copyright (c) 2005-2007 CollabNet.  All rights reserved.
+ * Copyright (c) 2005-2008 CollabNet.  All rights reserved.
  *
  * This software is licensed as described in the file COPYING, which
  * you should have received as part of this distribution.  The terms
@@ -24,6 +24,8 @@
 #include "svn_opt.h"
 #include "svn_ra.h"
 
+#include "private/svn_opt_private.h"
+
 #include "svn_private_config.h"
 
 #include <apr_network_io.h>
@@ -33,6 +35,7 @@
 static svn_opt_subcommand_t initialize_cmd,
                             synchronize_cmd,
                             copy_revprops_cmd,
+                            info_cmd,
                             help_cmd;
 
 enum {
@@ -45,21 +48,22 @@ enum {
   svnsync_opt_sync_username,
   svnsync_opt_sync_password,
   svnsync_opt_config_dir,
-  svnsync_opt_version
+  svnsync_opt_version,
+  svnsync_opt_trust_server_cert
 };
 
 #define SVNSYNC_OPTS_DEFAULT svnsync_opt_non_interactive, \
                              svnsync_opt_no_auth_cache, \
                              svnsync_opt_auth_username, \
                              svnsync_opt_auth_password, \
+                             svnsync_opt_trust_server_cert, \
                              svnsync_opt_source_username, \
                              svnsync_opt_source_password, \
                              svnsync_opt_sync_username, \
                              svnsync_opt_sync_password, \
-                             svnsync_opt_config_dir, \
-                             'q'
+                             svnsync_opt_config_dir
 
-static const svn_opt_subcommand_desc_t svnsync_cmd_table[] =
+static const svn_opt_subcommand_desc2_t svnsync_cmd_table[] =
   {
     { "initialize", initialize_cmd, { "init" },
       N_("usage: svnsync initialize DEST_URL SOURCE_URL\n"
@@ -71,17 +75,20 @@ static const svn_opt_subcommand_desc_t svnsync_cmd_table[] =
          "no committed revisions.  The destination repository must allow\n"
          "revision property changes.\n"
          "\n"
+         "If the source URL is not the root of a repository, only the\n"
+         "specified part of the repository will be synchronized.\n"
+         "\n"
          "You should not commit to, or make revision property changes in,\n"
          "the destination repository by any method other than 'svnsync'.\n"
          "In other words, the destination repository should be a read-only\n"
          "mirror of the source repository.\n"),
-      { SVNSYNC_OPTS_DEFAULT } },
+      { SVNSYNC_OPTS_DEFAULT, 'q' } },
     { "synchronize", synchronize_cmd, { "sync" },
       N_("usage: svnsync synchronize DEST_URL\n"
          "\n"
          "Transfer all pending revisions to the destination from the source\n"
          "with which it was initialized.\n"),
-      { SVNSYNC_OPTS_DEFAULT } },
+      { SVNSYNC_OPTS_DEFAULT, 'q' } },
     { "copy-revprops", copy_revprops_cmd, { 0 },
       N_("usage: svnsync copy-revprops DEST_URL [REV[:REV2]]\n"
          "\n"
@@ -97,6 +104,12 @@ static const svn_opt_subcommand_desc_t svnsync_cmd_table[] =
          "REV and REV2 must be revisions which were previously transferred\n"
          "to the destination.  You may use \"HEAD\" for either revision to\n"
          "mean \"the last revision transferred\".\n"),
+      { SVNSYNC_OPTS_DEFAULT, 'q' } },
+    { "info", info_cmd, { 0 },
+      N_("usage: svnsync info DEST_URL\n"
+         "\n"
+         "Print information about the synchronization destination repository\n"
+         "located at DEST_URL.\n"),
       { SVNSYNC_OPTS_DEFAULT } },
     { "help", help_cmd, { "?", "h" },
       N_("usage: svnsync help [SUBCOMMAND...]\n"
@@ -122,6 +135,10 @@ static const apr_getopt_option_t svnsync_options[] =
                        N_("specify a password ARG (deprecated;\n"
                           "                             "
                           "see --source-password and --sync-password)") },
+    {"trust-server-cert", svnsync_opt_trust_server_cert, 0,
+                       N_("accept unknown SSL server certificates without\n"
+                          "                             "
+                          "prompting (but only with '--non-interactive')") },
     {"source-username", svnsync_opt_source_username, 1,
                        N_("connect to source repository with username ARG") },
     {"source-password", svnsync_opt_source_password, 1,
@@ -143,6 +160,7 @@ static const apr_getopt_option_t svnsync_options[] =
 
 typedef struct {
   svn_boolean_t non_interactive;
+  svn_boolean_t trust_server_cert;
   svn_boolean_t no_auth_cache;
   svn_auth_baton_t *source_auth_baton;
   svn_auth_baton_t *sync_auth_baton;
@@ -274,7 +292,7 @@ typedef struct {
   /* initialize only */
   const char *from_url;
 
-  /* syncronize only */
+  /* synchronize only */
   svn_revnum_t committed_rev;
 
   /* copy-revprops only */
@@ -328,16 +346,9 @@ with_locked(svn_ra_session_t *session,
 static svn_error_t *
 open_tmp_file(apr_file_t **fp, void *callback_baton, apr_pool_t *pool)
 {
-  const char *path;
-
-  SVN_ERR(svn_io_temp_dir(&path, pool));
-
-  path = svn_path_join(path, "tempfile", pool);
-
-  SVN_ERR(svn_io_open_unique_file2(fp, NULL, path, ".tmp",
-                                   svn_io_file_del_on_close, pool));
-
-  return SVN_NO_ERROR;
+  return svn_io_open_unique_file3(fp, NULL, NULL,
+                                  svn_io_file_del_on_pool_cleanup,
+                                  pool, pool);
 }
 
 
@@ -363,7 +374,7 @@ check_if_session_is_at_repos_root(svn_ra_session_t *sess,
 }
 
 
-/* Remove the properties in TARGET_PROPS but not in SOURCE_PROPS from 
+/* Remove the properties in TARGET_PROPS but not in SOURCE_PROPS from
  * revision REV of the repository associated with RA session SESSION.
  *
  * All allocations will be done in a subpool of POOL.
@@ -401,21 +412,21 @@ remove_props_not_in_source(svn_ra_session_t *session,
 
 /* Filter callback function.
  * Takes a property name KEY, and is expected to return TRUE if the property
- * should be filtered out (ie. not be copied to the target list), or FALSE if 
- * not. 
+ * should be filtered out (ie. not be copied to the target list), or FALSE if
+ * not.
  */
 typedef svn_boolean_t (*filter_func_t)(const char *key);
 
 /* Make a new set of properties, by copying those properties in PROPS for which
  * the filter FILTER returns FALSE.
- * 
+ *
  * The number of filtered properties will be stored in FILTERED_COUNT.
  *
  * The returned set of properties is allocated from POOL.
  */
 static apr_hash_t *
-filter_props(int *filtered_count, apr_hash_t *props, 
-             filter_func_t filter, 
+filter_props(int *filtered_count, apr_hash_t *props,
+             filter_func_t filter,
              apr_pool_t *pool)
 {
   apr_hash_index_t *hi;
@@ -447,13 +458,13 @@ filter_props(int *filtered_count, apr_hash_t *props,
 }
 
 
-/* Write the set of revision properties REV_PROPS to revision REV to the 
+/* Write the set of revision properties REV_PROPS to revision REV to the
  * repository associated with RA session SESSION.
  *
  * All allocations will be done in a subpool of POOL.
  */
 static svn_error_t *
-write_revprops(int *filtered_count, 
+write_revprops(int *filtered_count,
                svn_ra_session_t *session,
                svn_revnum_t rev,
                apr_hash_t *rev_props,
@@ -472,7 +483,7 @@ write_revprops(int *filtered_count,
       svn_pool_clear(subpool);
       apr_hash_this(hi, &key, NULL, &val);
 
-      if (strncmp(key, SVNSYNC_PROP_PREFIX, 
+      if (strncmp(key, SVNSYNC_PROP_PREFIX,
                   sizeof(SVNSYNC_PROP_PREFIX) - 1) != 0)
         {
           SVN_ERR(svn_ra_change_rev_prop(session, rev, key, val, subpool));
@@ -490,7 +501,7 @@ write_revprops(int *filtered_count,
 
 
 static svn_error_t *
-log_properties_copied(svn_boolean_t syncprops_found, 
+log_properties_copied(svn_boolean_t syncprops_found,
                       svn_revnum_t rev,
                       apr_pool_t *pool)
 {
@@ -540,12 +551,12 @@ copy_revprops(svn_ra_session_t *from_session,
 
   /* Delete those properties that were in TARGET but not in SOURCE */
   if (sync)
-    SVN_ERR(remove_props_not_in_source(to_session, rev, 
+    SVN_ERR(remove_props_not_in_source(to_session, rev,
                                        rev_props, existing_props, pool));
 
   if (! quiet)
     SVN_ERR(log_properties_copied(filtered_count > 0, rev, pool));
-    
+
   svn_pool_destroy(subpool);
 
   return SVN_NO_ERROR;
@@ -624,7 +635,7 @@ do_initialize(svn_ra_session_t *to_session,
                        baton->config, pool));
   SVN_ERR(svn_ra_get_repos_root2(from_session, &root_url, pool));
 
-  /* If we're doing a partial replay, we have to check first if the server 
+  /* If we're doing a partial replay, we have to check first if the server
      supports this. */
   if (svn_path_is_child(root_url, baton->from_url, pool))
     {
@@ -683,7 +694,7 @@ initialize_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
   apr_array_header_t *targets;
   subcommand_baton_t *baton;
 
-  SVN_ERR(svn_opt_args_to_target_array2(&targets, os,
+  SVN_ERR(svn_opt__args_to_target_array(&targets, os,
                                         apr_array_make(pool, 0,
                                                        sizeof(const char *)),
                                         pool));
@@ -703,7 +714,7 @@ initialize_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
                              _("Path '%s' is not a URL"), from_url);
 
   baton = make_subcommand_baton(opt_baton, to_url, from_url, 0, 0, pool);
-  SVN_ERR(svn_ra_open3(&to_session, baton->to_url, NULL, 
+  SVN_ERR(svn_ra_open3(&to_session, baton->to_url, NULL,
                        &(baton->sync_callbacks), baton, baton->config, pool));
   SVN_ERR(check_if_session_is_at_repos_root(to_session, baton->to_url, pool));
   SVN_ERR(with_locked(to_session, do_initialize, baton, pool));
@@ -738,6 +749,11 @@ typedef struct {
   svn_boolean_t got_textdeltas;
   svn_revnum_t base_revision;
   svn_boolean_t quiet;
+  svn_boolean_t strip_mergeinfo;    /* Are we stripping svn:mergeinfo? */
+  svn_boolean_t migrate_svnmerge;   /* Are we converting svnmerge.py data? */
+  svn_boolean_t mergeinfo_stripped; /* Did we strip svn:mergeinfo? */
+  svn_boolean_t svnmerge_migrated;  /* Did we convert svnmerge.py data? */
+  svn_boolean_t svnmerge_blocked;   /* Was there any blocked svnmerge data? */
 } edit_baton_t;
 
 
@@ -966,6 +982,27 @@ change_file_prop(void *file_baton,
   if (svn_property_kind(NULL, name) != svn_prop_regular_kind)
     return SVN_NO_ERROR;
 
+  /* Maybe drop svn:mergeinfo.  */
+  if (eb->strip_mergeinfo && (strcmp(name, SVN_PROP_MERGEINFO) == 0))
+    {
+      eb->mergeinfo_stripped = TRUE;
+      return SVN_NO_ERROR;
+    }
+
+  /* Maybe drop (errantly set, as this is a file) svnmerge.py properties. */
+  if (eb->migrate_svnmerge && (strcmp(name, "svnmerge-integrated") == 0))
+    {
+      eb->svnmerge_migrated = TRUE;
+      return SVN_NO_ERROR;
+    }
+
+  /* Remember if we see any svnmerge-blocked properties.  (They really
+     shouldn't be here, as this is a file, but whatever...)  */
+  if (eb->migrate_svnmerge && (strcmp(name, "svnmerge-blocked") == 0))
+    {
+      eb->svnmerge_blocked = TRUE;
+    }
+
   return eb->wrapped_editor->change_file_prop(fb->wrapped_node_baton,
                                               name, value, pool);
 }
@@ -978,13 +1015,84 @@ change_dir_prop(void *dir_baton,
 {
   node_baton_t *db = dir_baton;
   edit_baton_t *eb = db->edit_baton;
+  svn_string_t *real_value = (svn_string_t *)value;
 
-  /* only regular properties can pass over libsvn_ra */
+  /* Only regular properties can pass over libsvn_ra */
   if (svn_property_kind(NULL, name) != svn_prop_regular_kind)
     return SVN_NO_ERROR;
 
+  /* Maybe drop svn:mergeinfo.  */
+  if (eb->strip_mergeinfo && (strcmp(name, SVN_PROP_MERGEINFO) == 0))
+    {
+      eb->mergeinfo_stripped = TRUE;
+      return SVN_NO_ERROR;
+    }
+
+  /* Maybe convert svnmerge-integrated data into svn:mergeinfo.  (We
+     ignore svnmerge-blocked for now.) */
+  /* ### FIXME: Consult the mirror repository's HEAD prop values and
+     ### merge svn:mergeinfo, svnmerge-integrated, and svnmerge-blocked. */
+  if (eb->migrate_svnmerge && (strcmp(name, "svnmerge-integrated") == 0))
+    {
+      if (value)
+        {
+          /* svnmerge-integrated differs from svn:mergeinfo in a pair
+             of ways.  First, it can use tabs, newlines, or spaces to
+             delimit source information.  Secondly, the source paths
+             are relative URLs, whereas svn:mergeinfo uses relative
+             paths (not URI-encoded). */
+          svn_error_t *err;
+          svn_stringbuf_t *mergeinfo_buf = svn_stringbuf_create("", pool);
+          svn_mergeinfo_t mergeinfo;
+          int i;
+          apr_array_header_t *sources =
+            svn_cstring_split(value->data, " \t\n", TRUE, pool);
+
+          for (i = 0; i < sources->nelts; i++)
+            {
+              const char *rel_path;
+              apr_array_header_t *path_revs =
+                svn_cstring_split(APR_ARRAY_IDX(sources, i, const char *),
+                                  ":", TRUE, pool);
+
+              /* ### TODO: Warn? */
+              if (path_revs->nelts != 2)
+                continue;
+
+              /* Append this source's mergeinfo data. */
+              rel_path = APR_ARRAY_IDX(path_revs, 0, const char *);
+              rel_path = svn_path_uri_decode(rel_path, pool);
+              svn_stringbuf_appendcstr(mergeinfo_buf, rel_path);
+              svn_stringbuf_appendcstr(mergeinfo_buf, ":");
+              svn_stringbuf_appendcstr(mergeinfo_buf,
+                                       APR_ARRAY_IDX(path_revs, 1,
+                                                     const char *));
+              svn_stringbuf_appendcstr(mergeinfo_buf, "\n");
+            }
+
+          /* Try to parse the mergeinfo string we've created, just to
+             check for bogosity.  If all goes well, we'll unparse it
+             again and use that as our property value.  */
+          err = svn_mergeinfo_parse(&mergeinfo, mergeinfo_buf->data, pool);
+          if (err)
+            {
+              svn_error_clear(err);
+              return SVN_NO_ERROR;
+            }
+          SVN_ERR(svn_mergeinfo_to_string(&real_value, mergeinfo, pool));
+        }
+      name = SVN_PROP_MERGEINFO;
+      eb->svnmerge_migrated = TRUE;
+    }
+
+  /* Remember if we see any svnmerge-blocked properties. */
+  if (eb->migrate_svnmerge && (strcmp(name, "svnmerge-blocked") == 0))
+    {
+      eb->svnmerge_blocked = TRUE;
+    }
+
   return eb->wrapped_editor->change_dir_prop(db->wrapped_node_baton,
-                                             name, value, pool);
+                                             name, real_value, pool);
 }
 
 static svn_error_t *
@@ -1012,6 +1120,18 @@ close_edit(void *edit_baton,
     {
       if (eb->got_textdeltas)
         SVN_ERR(svn_cmdline_printf(pool, "\n"));
+      if (eb->mergeinfo_stripped)
+        SVN_ERR(svn_cmdline_printf(pool,
+                                   "NOTE: Dropped Subversion mergeinfo "
+                                   "from this revision.\n"));
+      if (eb->svnmerge_migrated)
+        SVN_ERR(svn_cmdline_printf(pool,
+                                   "NOTE: Migrated 'svnmerge-integrated' in "
+                                   "this revision.\n"));
+      if (eb->svnmerge_blocked)
+        SVN_ERR(svn_cmdline_printf(pool,
+                                   "NOTE: Saw 'svnmerge-blocked' in this "
+                                   "revision (but didn't migrate it).\n"));
     }
 
   return eb->wrapped_editor->close_edit(eb->wrapped_edit_baton, pool);
@@ -1060,6 +1180,25 @@ get_sync_editor(const svn_delta_editor_t *wrapped_editor,
   eb->to_url = to_url;
   eb->quiet = quiet;
 
+  if (getenv("SVNSYNC_UNSUPPORTED_STRIP_MERGEINFO"))
+    {
+      eb->strip_mergeinfo = TRUE;
+    }
+  if (getenv("SVNSYNC_UNSUPPORTED_MIGRATE_SVNMERGE"))
+    {
+      /* Current we can't merge property values.  That's only possible
+         if all the properties to be merged were always modified in
+         exactly the same revisions, or if we allow ourselves to
+         lookup the current state of properties in the sync
+         destination.  So for now, migrating svnmerge.py data implies
+         stripping pre-existing svn:mergeinfo. */
+      /* ### FIXME: Do a real migration by consulting the mirror
+         ### repository's HEAD propvalues and merging svn:mergeinfo,
+         ### svnmerge-integrated, and svnmerge-blocked together. */
+      eb->migrate_svnmerge = TRUE;
+      eb->strip_mergeinfo = TRUE;
+    }
+
   *editor = tree_editor;
   *edit_baton = eb;
 
@@ -1140,7 +1279,7 @@ typedef struct {
 /* Return a replay baton allocated from POOL and populated with
    data from the provided parameters. */
 static replay_baton_t *
-make_replay_baton(svn_ra_session_t *from_session, 
+make_replay_baton(svn_ra_session_t *from_session,
                   svn_ra_session_t *to_session,
                   subcommand_baton_t *sb, apr_pool_t *pool)
 {
@@ -1152,7 +1291,7 @@ make_replay_baton(svn_ra_session_t *from_session,
 }
 
 /* Filter out svn:date and svn:author properties. */
-static svn_boolean_t 
+static svn_boolean_t
 filter_exclude_date_author_sync(const char *key)
 {
   if (strcmp(key, SVN_PROP_REVISION_AUTHOR) == 0)
@@ -1167,7 +1306,7 @@ filter_exclude_date_author_sync(const char *key)
 }
 
 /* Filter out all properties except svn:date and svn:author */
-static svn_boolean_t 
+static svn_boolean_t
 filter_include_date_author_sync(const char *key)
 {
   return ! filter_exclude_date_author_sync(key);
@@ -1195,7 +1334,7 @@ filter_include_log(const char *key)
 /* Callback function for svn_ra_replay_range, invoked when starting to parse
  * a replay report.
  */
-static svn_error_t * 
+static svn_error_t *
 replay_rev_started(svn_revnum_t revision,
                    void *replay_baton,
                    const svn_delta_editor_t **editor,
@@ -1260,7 +1399,7 @@ replay_rev_started(svn_revnum_t revision,
      over the RA interface, so we need an editor that's smart enough
      to filter those out for us.  */
   SVN_ERR(get_sync_editor(commit_editor, commit_baton, revision - 1,
-                          rb->sb->to_url, rb->sb->quiet, 
+                          rb->sb->to_url, rb->sb->quiet,
                           &sync_editor, &sync_baton, pool));
 
   SVN_ERR(svn_delta_get_cancellation_editor(check_cancel, NULL,
@@ -1277,7 +1416,7 @@ replay_rev_started(svn_revnum_t revision,
 /* Callback function for svn_ra_replay_range, invoked when finishing parsing
  * a replay report.
  */
-static svn_error_t * 
+static svn_error_t *
 replay_rev_finished(svn_revnum_t revision,
                     void *replay_baton,
                     const svn_delta_editor_t *editor,
@@ -1303,7 +1442,7 @@ replay_rev_finished(svn_revnum_t revision,
                               subpool));
 
 
-  /* Ok, we're done with the data, now we just need to copy the remaining 
+  /* Ok, we're done with the data, now we just need to copy the remaining
      'svn:date' and 'svn:author' revprops and we're all set.
      If the server doesn't support revprops-in-a-commit, we still have to
      set all revision properties except svn:log. */
@@ -1312,11 +1451,11 @@ replay_rev_finished(svn_revnum_t revision,
                             ? filter_include_date_author_sync
                             : filter_exclude_log),
                           subpool);
-  SVN_ERR(write_revprops(&filtered_count, rb->to_session, revision, filtered, 
+  SVN_ERR(write_revprops(&filtered_count, rb->to_session, revision, filtered,
                          subpool));
 
   /* Remove all extra properties in TARGET. */
-  SVN_ERR(remove_props_not_in_source(rb->to_session, revision, 
+  SVN_ERR(remove_props_not_in_source(rb->to_session, revision,
                                      rev_props, existing_props, subpool));
 
   svn_pool_clear(subpool);
@@ -1412,7 +1551,7 @@ do_synchronize(svn_ra_session_t *to_session,
           if (copying > last_merged)
             {
               SVN_ERR(copy_revprops(from_session, to_session,
-                                    to_latest, TRUE, baton->quiet, 
+                                    to_latest, TRUE, baton->quiet,
                                     pool));
               last_merged = copying;
               last_merged_rev = svn_string_create
@@ -1455,21 +1594,21 @@ do_synchronize(svn_ra_session_t *to_session,
   /* Ok, so there are new revisions, iterate over them copying them
      into the destination repository. */
   rb = make_replay_baton(from_session, to_session, baton, pool);
-    
+
   /* For compatibility with older svnserve versions, check first if we
      support adding revprops to the commit. */
-  SVN_ERR(svn_ra_has_capability(rb->to_session, 
+  SVN_ERR(svn_ra_has_capability(rb->to_session,
                                 &rb->has_commit_revprops_capability,
                                 SVN_RA_CAPABILITY_COMMIT_REVPROPS,
                                 pool));
 
   start_revision = atol(last_merged_rev->data) + 1;
   end_revision = from_latest;
-  
+
   SVN_ERR(check_cancel(NULL));
 
-  SVN_ERR(svn_ra_replay_range(from_session, start_revision, end_revision, 
-                              0, TRUE, replay_rev_started, 
+  SVN_ERR(svn_ra_replay_range(from_session, start_revision, end_revision,
+                              0, TRUE, replay_rev_started,
                               replay_rev_finished, rb, pool));
 
   return SVN_NO_ERROR;
@@ -1486,7 +1625,7 @@ synchronize_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
   subcommand_baton_t *baton;
   const char *to_url;
 
-  SVN_ERR(svn_opt_args_to_target_array2(&targets, os,
+  SVN_ERR(svn_opt__args_to_target_array(&targets, os,
                                         apr_array_make(pool, 0,
                                                        sizeof(const char *)),
                                         pool));
@@ -1527,7 +1666,7 @@ do_copy_revprops(svn_ra_session_t *to_session,
   svn_revnum_t i;
   svn_revnum_t step = 1;
 
-  SVN_ERR(open_source_session(&from_session, &last_merged_rev, 
+  SVN_ERR(open_source_session(&from_session, &last_merged_rev,
                               to_session,
                               &(baton->source_callbacks), baton->config,
                               baton, pool));
@@ -1583,7 +1722,7 @@ copy_revprops_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
 
   /* If there are two args, the last one is a revision range.  We'll
      effectively pop it from the end of the list.  Why?  Because
-     svn_opt_args_to_target_array2() does waaaaay too many useful
+     svn_opt__args_to_target_array() does waaaaay too many useful
      things for us not to use it.  */
   if (os->argc - os->ind == 2)
     {
@@ -1638,7 +1777,7 @@ copy_revprops_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
         }
     }
 
-  SVN_ERR(svn_opt_args_to_target_array2(&targets, os,
+  SVN_ERR(svn_opt__args_to_target_array(&targets, os,
                                         apr_array_make(pool, 1,
                                                        sizeof(const char *)),
                                         pool));
@@ -1662,6 +1801,68 @@ copy_revprops_cmd(apr_getopt_t *os, void *b, apr_pool_t *pool)
 }
 
 
+
+/*** `svnsync info' ***/
+
+
+/* SUBCOMMAND: info */
+static svn_error_t *
+info_cmd(apr_getopt_t *os, void *b, apr_pool_t * pool)
+{
+  svn_ra_session_t *to_session;
+  opt_baton_t *opt_baton = b;
+  apr_array_header_t *targets;
+  subcommand_baton_t *baton;
+  const char *to_url;
+  svn_string_t *from_url, *from_uuid, *last_merged_rev;
+
+  SVN_ERR(svn_opt__args_to_target_array(&targets, os,
+                                        apr_array_make(pool, 0,
+                                                       sizeof(const char *)),
+                                        pool));
+  if (targets->nelts < 1)
+    return svn_error_create(SVN_ERR_CL_INSUFFICIENT_ARGS, 0, NULL);
+  if (targets->nelts > 1)
+    return svn_error_create(SVN_ERR_CL_ARG_PARSING_ERROR, 0, NULL);
+
+  /* Get the mirror repository URL, and verify that it is URL-ish. */
+  to_url = APR_ARRAY_IDX(targets, 0, const char *);
+  if (! svn_path_is_url(to_url))
+    return svn_error_createf(SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                             _("Path '%s' is not a URL"), to_url);
+
+  /* Open an RA session to the mirror repository URL. */
+  baton = make_subcommand_baton(opt_baton, to_url, NULL, 0, 0, pool);
+  SVN_ERR(svn_ra_open3(&to_session, baton->to_url, NULL,
+                       &(baton->sync_callbacks), baton, baton->config, pool));
+  SVN_ERR(check_if_session_is_at_repos_root(to_session, baton->to_url, pool));
+
+  /* Verify that the repos has been initialized for synchronization. */
+  SVN_ERR(svn_ra_rev_prop(to_session, 0, SVNSYNC_PROP_FROM_URL,
+                          &from_url, pool));
+  if (! from_url)
+    return svn_error_createf
+      (SVN_ERR_BAD_URL, NULL,
+       _("Repository '%s' is not initialized for synchronization"), to_url);
+
+  /* Fetch more of the magic properties, which are the source of our info. */
+  SVN_ERR(svn_ra_rev_prop(to_session, 0, SVNSYNC_PROP_FROM_UUID,
+                          &from_uuid, pool));
+  SVN_ERR(svn_ra_rev_prop(to_session, 0, SVNSYNC_PROP_LAST_MERGED_REV,
+                          &last_merged_rev, pool));
+
+  /* Print the info. */
+  SVN_ERR(svn_cmdline_printf(pool, _("Source URL: %s\n"), from_url->data));
+  if (from_uuid)
+    SVN_ERR(svn_cmdline_printf(pool, _("Source Repository UUID: %s\n"),
+                               from_uuid->data));
+  if (last_merged_rev)
+    SVN_ERR(svn_cmdline_printf(pool, _("Last Merged Revision: %s\n"),
+                               last_merged_rev->data));
+  return SVN_NO_ERROR;
+}
+
+
 
 /*** `svnsync help' ***/
 
@@ -1687,11 +1888,11 @@ help_cmd(apr_getopt_t *os, void *baton, apr_pool_t *pool)
 
   SVN_ERR(svn_ra_print_modules(version_footer, pool));
 
-  SVN_ERR(svn_opt_print_help(os, "svnsync",
-                             opt_baton ? opt_baton->version : FALSE,
-                             FALSE, version_footer->data, header,
-                             svnsync_cmd_table, svnsync_options, NULL,
-                             pool));
+  SVN_ERR(svn_opt_print_help3(os, "svnsync",
+                              opt_baton ? opt_baton->version : FALSE,
+                              FALSE, version_footer->data, header,
+                              svnsync_cmd_table, svnsync_options, NULL,
+                              NULL, pool));
 
   return SVN_NO_ERROR;
 }
@@ -1703,7 +1904,7 @@ help_cmd(apr_getopt_t *os, void *baton, apr_pool_t *pool)
 int
 main(int argc, const char *argv[])
 {
-  const svn_opt_subcommand_desc_t *subcommand = NULL;
+  const svn_opt_subcommand_desc2_t *subcommand = NULL;
   apr_array_header_t *received_opts;
   opt_baton_t opt_baton;
   svn_config_t *config;
@@ -1769,6 +1970,10 @@ main(int argc, const char *argv[])
             opt_baton.non_interactive = TRUE;
             break;
 
+          case svnsync_opt_trust_server_cert:
+            opt_baton.trust_server_cert = TRUE;
+            break;
+
           case svnsync_opt_no_auth_cache:
             opt_baton.no_auth_cache = TRUE;
             break;
@@ -1824,7 +2029,7 @@ main(int argc, const char *argv[])
     }
 
   if (opt_baton.help)
-    subcommand = svn_opt_get_canonical_subcommand(svnsync_cmd_table, "help");
+    subcommand = svn_opt_get_canonical_subcommand2(svnsync_cmd_table, "help");
 
   /* Disallow the mixing --username/password with their --source- and
      --sync- variants.  Treat "--username FOO" as "--source-username
@@ -1855,6 +2060,15 @@ main(int argc, const char *argv[])
   opt_baton.sync_username = sync_username;
   opt_baton.sync_password = sync_password;
 
+  /* --trust-server-cert can only be used with --non-interactive */
+  if (opt_baton.trust_server_cert && !opt_baton.non_interactive)
+    {
+      err = svn_error_create(SVN_ERR_CL_ARG_PARSING_ERROR, NULL,
+                             _("--trust-server-cert requires "
+                               "--non-interactive"));
+      return svn_cmdline_handle_exit_error(err, pool, "svnsync: ");
+    }
+
   err = svn_config_ensure(opt_baton.config_dir, pool);
   if (err)
     return svn_cmdline_handle_exit_error(err, pool, "synsync: ");
@@ -1866,7 +2080,7 @@ main(int argc, const char *argv[])
           if (opt_baton.version)
             {
               /* Use the "help" subcommand to handle "--version". */
-              static const svn_opt_subcommand_desc_t pseudo_cmd =
+              static const svn_opt_subcommand_desc2_t pseudo_cmd =
                 { "--version", help_cmd, {0}, "",
                   {svnsync_opt_version,  /* must accept its own option */
                   } };
@@ -1883,8 +2097,8 @@ main(int argc, const char *argv[])
       else
         {
           const char *first_arg = os->argv[os->ind++];
-          subcommand = svn_opt_get_canonical_subcommand(svnsync_cmd_table,
-                                                        first_arg);
+          subcommand = svn_opt_get_canonical_subcommand2(svnsync_cmd_table,
+                                                         first_arg);
           if (subcommand == NULL)
             {
               help_cmd(NULL, NULL, pool);
@@ -1901,11 +2115,12 @@ main(int argc, const char *argv[])
       if (opt_id == 'h' || opt_id == '?')
         continue;
 
-      if (! svn_opt_subcommand_takes_option(subcommand, opt_id))
+      if (! svn_opt_subcommand_takes_option3(subcommand, opt_id, NULL))
         {
           const char *optstr;
           const apr_getopt_option_t *badopt =
-            svn_opt_get_option_from_code(opt_id, svnsync_options);
+            svn_opt_get_option_from_code2(opt_id, svnsync_options, subcommand,
+                                          pool);
           svn_opt_format_option(&optstr, badopt, FALSE, pool);
           if (subcommand->name[0] == '-')
             {
@@ -1958,25 +2173,27 @@ main(int argc, const char *argv[])
   apr_signal(SIGXFSZ, SIG_IGN);
 #endif
 
-  err = svn_cmdline_setup_auth_baton(&opt_baton.source_auth_baton,
-                                     opt_baton.non_interactive,
-                                     opt_baton.source_username,
-                                     opt_baton.source_password,
-                                     opt_baton.config_dir,
-                                     opt_baton.no_auth_cache,
-                                     config,
-                                     check_cancel, NULL,
-                                     pool);
+  err = svn_cmdline_create_auth_baton(&opt_baton.source_auth_baton,
+                                      opt_baton.non_interactive,
+                                      opt_baton.source_username,
+                                      opt_baton.source_password,
+                                      opt_baton.config_dir,
+                                      opt_baton.no_auth_cache,
+                                      opt_baton.trust_server_cert,
+                                      config,
+                                      check_cancel, NULL,
+                                      pool);
   if (! err)
-    err = svn_cmdline_setup_auth_baton(&opt_baton.sync_auth_baton,
-                                       opt_baton.non_interactive,
-                                       opt_baton.sync_username,
-                                       opt_baton.sync_password,
-                                       opt_baton.config_dir,
-                                       opt_baton.no_auth_cache,
-                                       config,
-                                       check_cancel, NULL,
-                                       pool);
+    err = svn_cmdline_create_auth_baton(&opt_baton.sync_auth_baton,
+                                        opt_baton.non_interactive,
+                                        opt_baton.sync_username,
+                                        opt_baton.sync_password,
+                                        opt_baton.config_dir,
+                                        opt_baton.no_auth_cache,
+                                        opt_baton.trust_server_cert,
+                                        config,
+                                        check_cancel, NULL,
+                                        pool);
   if (! err)
     err = (*subcommand->cmd_func)(os, &opt_baton, pool);
   if (err)
