@@ -4,16 +4,19 @@ Driver for running the tests on Windows.
 For a list of options, run this script with the --help option.
 """
 
-# $HeadURL: http://svn.apache.org/repos/asf/subversion/branches/1.5.x/win-tests.py $
-# $LastChangedDate: 2009-02-24 21:58:43 +0000 (Tue, 24 Feb 2009) $
-# $LastChangedBy: hwright $
-# $LastChangedRevision: 876184 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.6.x/win-tests.py $
+# $LastChangedRevision: 35009 $
 
 import os, sys
 import filecmp
 import shutil
 import traceback
-import ConfigParser
+try:
+  # Python >=3.0
+  import configparser
+except ImportError:
+  # Python <3.0
+  import ConfigParser as configparser
 import string
 import random
 
@@ -26,34 +29,39 @@ except AttributeError:
 def _usage_exit():
   "print usage, exit the script"
 
-  print "Driver for running the tests on Windows."
-  print "Usage: python win-tests.py [option] [test-path]"
-  print
-  print "Valid options:"
-  print "  -r, --release          : test the Release configuration"
-  print "  -d, --debug            : test the Debug configuration (default)"
-  print "  --bin=PATH             : use the svn binaries installed in PATH"
-  print "  -u URL, --url=URL      : run ra_dav or ra_svn tests against URL;"
-  print "                           will start svnserve for ra_svn tests"
-  print "  -v, --verbose          : talk more"
-  print "  -f, --fs-type=type     : filesystem type to use (fsfs is default)"
-  print "  -c, --cleanup          : cleanup after running a test"
+  print("Driver for running the tests on Windows.")
+  print("Usage: python win-tests.py [option] [test-path]")
+  print("")
+  print("Valid options:")
+  print("  -r, --release          : test the Release configuration")
+  print("  -d, --debug            : test the Debug configuration (default)")
+  print("  --bin=PATH             : use the svn binaries installed in PATH")
+  print("  -u URL, --url=URL      : run ra_dav or ra_svn tests against URL;")
+  print("                           will start svnserve for ra_svn tests")
+  print("  -v, --verbose          : talk more")
+  print("  -f, --fs-type=type     : filesystem type to use (fsfs is default)")
+  print("  -c, --cleanup          : cleanup after running a test")
 
-  print "  --svnserve-args=list   : comma-separated list of arguments for"
-  print "                           svnserve"
-  print "                           default is '-d,-r,<test-path-root>'"
-  print "  --asp.net-hack         : use '_svn' instead of '.svn' for the admin"
-  print "                           dir name"
-  print "  --httpd-dir            : location where Apache HTTPD is installed"
-  print "  --httpd-port           : port for Apache HTTPD; random port number"
-  print "                           will be used, if not specified"
-  print "  --http-library         : dav library to use, neon (default) or serf"
-  print "  --list                 : print test doc strings only"
-  print "  --enable-sasl          : enable Cyrus SASL authentication for"
-  print "                           svnserve"
-  print "  -p, --parallel         : run multiple tests in parallel"
-  print "  --server-minor-version : the minor version of the server being"
-  print "                           tested"
+  print("  --svnserve-args=list   : comma-separated list of arguments for")
+  print("                           svnserve")
+  print("                           default is '-d,-r,<test-path-root>'")
+  print("  --asp.net-hack         : use '_svn' instead of '.svn' for the admin")
+  print("                           dir name")
+  print("  --httpd-dir            : location where Apache HTTPD is installed")
+  print("  --httpd-port           : port for Apache HTTPD; random port number")
+  print("                           will be used, if not specified")
+  print("  --httpd-daemon         : Run Apache httpd as daemon")
+  print("  --httpd-service        : Run Apache httpd as Windows service (default)")
+  print("  --http-library         : dav library to use, neon (default) or serf")
+  print("  --list                 : print test doc strings only")
+  print("  --enable-sasl          : enable Cyrus SASL authentication for")
+  print("                           svnserve")
+  print("  -p, --parallel         : run multiple tests in parallel")
+  print("  --server-minor-version : the minor version of the server being")
+  print("                           tested")
+  print(" --config-file           : Configuration file for tests")
+  print(" --fsfs-sharding         : Specify shard size (for fsfs)")
+  print(" --fsfs-packing          : Run 'svnadmin pack' automatically")
 
   sys.exit(0)
 
@@ -65,14 +73,13 @@ sys.path.insert(1, 'build')
 
 import gen_win
 version_header = os.path.join('subversion', 'include', 'svn_version.h')
-cp = ConfigParser.ConfigParser()
+cp = configparser.ConfigParser()
 cp.read('gen-make.opts')
 gen_obj = gen_win.GeneratorBase('build.conf', version_header,
                                 cp.items('options'))
 all_tests = gen_obj.test_progs + gen_obj.bdb_test_progs \
           + gen_obj.scripts + gen_obj.bdb_scripts
-client_tests = filter(lambda x: x.startswith(CMDLINE_TEST_SCRIPT_PATH),
-                      all_tests)
+client_tests = [x for x in all_tests if x.startswith(CMDLINE_TEST_SCRIPT_PATH)]
 
 svn_dlls = []
 for section in gen_obj.sections.values():
@@ -83,11 +90,13 @@ for section in gen_obj.sections.values():
 opts, args = my_getopt(sys.argv[1:], 'hrdvcpu:f:',
                        ['release', 'debug', 'verbose', 'cleanup', 'url=',
                         'svnserve-args=', 'fs-type=', 'asp.net-hack',
-                        'httpd-dir=', 'httpd-port=', 'http-library=', 'help',
+                        'httpd-dir=', 'httpd-port=', 'httpd-daemon',
+                        'httpd-server', 'http-library=', 'help',
+                        'fsfs-packing', 'fsfs-sharding=',
                         'list', 'enable-sasl', 'bin=', 'parallel',
-                        'server-minor-version='])
+                        'config-file=', 'server-minor-version='])
 if len(args) > 1:
-  print 'Warning: non-option arguments after the first one will be ignored'
+  print('Warning: non-option arguments after the first one will be ignored')
 
 # Interpret the options and set parameters
 base_url, fs_type, verbose, cleanup = None, None, None, None
@@ -98,12 +107,16 @@ run_svnserve = None
 svnserve_args = None
 run_httpd = None
 httpd_port = None
+httpd_service = None
 http_library = 'neon'
 list_tests = None
 enable_sasl = None
 svn_bin = None
 parallel = None
+fsfs_sharding = None
+fsfs_packing = None
 server_minor_version = None
+config_file = None
 
 for opt, val in opts:
   if opt in ('-h', '--help'):
@@ -130,8 +143,16 @@ for opt, val in opts:
     run_httpd = 1
   elif opt == '--httpd-port':
     httpd_port = int(val)
+  elif opt == '--httpd-daemon':
+    httpd_service = 0
+  elif opt == '--httpd-service':
+    httpd_service = 1
   elif opt == '--http-library':
     http_library = val
+  elif opt == '--fsfs-sharding':
+    fsfs_sharding = int(val)
+  elif opt == '--fsfs-packing':
+    fsfs_packing = 1
   elif opt == '--list':
     list_tests = 1
   elif opt == '--enable-sasl':
@@ -143,6 +164,8 @@ for opt, val in opts:
     svn_bin = val
   elif opt in ('-p', '--parallel'):
     parallel = 1
+  elif opt in ('--config-file'):
+    config_file = val
 
 # Calculate the source and test directory names
 abs_srcdir = os.path.abspath("")
@@ -188,12 +211,12 @@ def create_target_dir(dirname):
   tgt_dir = os.path.join(abs_builddir, dirname)
   if not os.path.exists(tgt_dir):
     if verbose:
-      print "mkdir:", tgt_dir
+      print("mkdir: %s" % tgt_dir)
     os.makedirs(tgt_dir)
 
 def copy_changed_file(src, tgt):
   if not os.path.isfile(src):
-    print 'Could not find ' + src
+    print('Could not find ' + src)
     sys.exit(1)
   if os.path.isdir(tgt):
     tgt = os.path.join(tgt, os.path.basename(src))
@@ -201,12 +224,12 @@ def copy_changed_file(src, tgt):
     assert os.path.isfile(tgt)
     if filecmp.cmp(src, tgt):
       if verbose:
-        print "same:", src
-        print " and:", tgt
+        print("same: %s" % src)
+        print(" and: %s" % tgt)
       return 0
   if verbose:
-    print "copy:", src
-    print "  to:", tgt
+    print("copy: %s" % src)
+    print("  to: %s" % tgt)
   shutil.copy(src, tgt)
   return 1
 
@@ -272,14 +295,14 @@ def locate_libs():
 
 def fix_case(path):
     path = os.path.normpath(path)
-    parts = string.split(path, os.path.sep)
-    drive = string.upper(parts[0])
+    parts = path.split(os.path.sep)
+    drive = parts[0].upper()
     parts = parts[1:]
     path = drive + os.path.sep
     for part in parts:
         dirs = os.listdir(path)
         for dir in dirs:
-            if string.lower(dir) == string.lower(part):
+            if dir.lower() == part.lower():
                 path = os.path.join(path, dir)
                 break
     return path
@@ -310,11 +333,11 @@ class Svnserve:
       args = [self.name, '-d', '-r', self.root]
     else:
       args = [self.name] + self.args
-    print 'Starting', self.kind, self.name
+    print('Starting %s %s' % (self.kind, self.name))
     try:
       import win32process
       import win32con
-      args = ' '.join(map(lambda x: self._quote(x), args))
+      args = ' '.join([self._quote(x) for x in args])
       self.proc_handle = (
         win32process.CreateProcess(self._quote(self.path), args,
                                    None, None, 0,
@@ -327,26 +350,28 @@ class Svnserve:
     if self.proc_handle is not None:
       try:
         import win32process
-        print 'Stopping', self.name
+        print('Stopping %s' % self.name)
         win32process.TerminateProcess(self.proc_handle, 0)
         return
       except ImportError:
         pass
-    print 'Svnserve.stop not implemented'
+    print('Svnserve.stop not implemented')
 
 class Httpd:
   "Run httpd for DAV tests"
-  def __init__(self, abs_httpd_dir, abs_objdir, abs_builddir, httpd_port):
+  def __init__(self, abs_httpd_dir, abs_objdir, abs_builddir, httpd_port, service):
     self.name = 'apache.exe'
     self.httpd_port = httpd_port
     self.httpd_dir = abs_httpd_dir
+    self.service = service
+    self.proc_handle = None
     self.path = os.path.join(self.httpd_dir, 'bin', self.name)
 
     if not os.path.exists(self.path):
       self.name = 'httpd.exe'
       self.path = os.path.join(self.httpd_dir, 'bin', self.name)
       if not os.path.exists(self.path):
-        raise RuntimeError, "Could not find a valid httpd binary!"
+        raise RuntimeError("Could not find a valid httpd binary!")
 
     self.root_dir = os.path.join(CMDLINE_TEST_SCRIPT_NATIVE_PATH, 'httpd')
     self.root = os.path.join(abs_builddir, self.root_dir)
@@ -359,8 +384,12 @@ class Httpd:
     self.abs_builddir = abs_builddir
     self.abs_objdir = abs_objdir
     self.service_name = 'svn-test-httpd-' + str(httpd_port)
-    self.httpd_args = [self.name, '-n', self._quote(self.service_name),
-                       '-f', self._quote(self.httpd_config)]
+
+    if self.service:
+      self.httpd_args = [self.name, '-n', self._quote(self.service_name),
+                         '-f', self._quote(self.httpd_config)]
+    else:
+      self.httpd_args = [self.name, '-f', self._quote(self.httpd_config)]
 
     create_target_dir(self.root_dir)
 
@@ -465,17 +494,56 @@ class Httpd:
       '</Location>\n'
 
   def start(self):
-    "Install and start HTTPD service"
-    print 'Installing service', self.service_name
-    os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'install'])
-    print 'Starting service', self.service_name
-    os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'start'])
+    if self.service:
+      self._start_service()
+    else:
+      self._start_daemon()
 
   def stop(self):
-    "Stop and unintall HTTPD service"
+    if self.service:
+      self._stop_service()
+    else:
+      self._stop_daemon()
+
+  def _start_service(self):
+    "Install and start HTTPD service"
+    print('Installing service %s' % self.service_name)
+    os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'install'])
+    print('Starting service %s' % self.service_name)
+    os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'start'])
+
+  def _stop_service(self):
+    "Stop and uninstall HTTPD service"
     os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'stop'])
     os.spawnv(os.P_WAIT, self.path, self.httpd_args + ['-k', 'uninstall'])
 
+  def _start_daemon(self):
+    "Start HTTPD as daemon"
+    print('Starting httpd as daemon')
+    print(self.httpd_args)
+    try:
+      import win32process
+      import win32con
+      self.proc_handle = (
+        win32process.CreateProcess(self._quote(self.path), self.httpd_args,
+                                   None, None, 0,
+                                   win32con.CREATE_NEW_CONSOLE,
+                                   None, None, win32process.STARTUPINFO()))[0]
+    except ImportError:
+      os.spawnv(os.P_NOWAIT, self.path, self.httpd_args)
+
+  def _stop_daemon(self):
+    "Stop the HTTPD daemon"
+    if self.proc_handle is not None:
+      try:
+        import win32process
+        print('Stopping %s' % self.name)
+        win32process.TerminateProcess(self.proc_handle, 0)
+        return
+      except ImportError:
+        pass
+    print('Httpd.stop_daemon not implemented')
+
 # Move the binaries to the test directory
 locate_libs()
 if create_dirs:
@@ -502,21 +570,23 @@ if run_svnserve:
   daemon = Svnserve(svnserve_args, objdir, abs_objdir, abs_builddir)
 
 if run_httpd:
-  daemon = Httpd(abs_httpd_dir, abs_objdir, abs_builddir, httpd_port)
+  daemon = Httpd(abs_httpd_dir, abs_objdir, abs_builddir, httpd_port,
+                 httpd_service)
 
 # Start service daemon, if any
 if daemon:
   daemon.start()
 
-print 'Testing', objdir, 'configuration on', repo_loc
+print('Testing %s configuration on %s' % (objdir, repo_loc))
 sys.path.insert(0, os.path.join(abs_srcdir, 'build'))
 import run_tests
 th = run_tests.TestHarness(abs_srcdir, abs_builddir,
                            os.path.join(abs_builddir, log),
                            base_url, fs_type, http_library,
                            server_minor_version, 1, cleanup,
-                           enable_sasl, parallel, list_tests,
-                           svn_bin)
+                           enable_sasl, parallel, config_file,
+                           fsfs_sharding, fsfs_packing,
+                           list_tests, svn_bin)
 old_cwd = os.getcwd()
 try:
   os.chdir(abs_builddir)
@@ -536,7 +606,7 @@ for tgt in copied_execs:
   try:
     if os.path.isfile(tgt):
       if verbose:
-        print "kill:", tgt
+        print("kill: %s" % tgt)
       os.unlink(tgt)
   except:
     traceback.print_exc(file=sys.stdout)
